"""
Usage

>>> from rpw import db
>>> levels = db.Collector(of_category='Levels', is_type=True)
>>> walls = db.Collector(of_class='Wall', where=lambda x: x.parameters['Length'] > 5)
>>> desks = db.Collector(of_class='FamilyInstance', level='Level 1')

Note:
    As of June 2017, these are the filters that have been implemented:

    | ``ElementCategoryFilter`` = ``of_category``
    | ``ElementClassFilter`` = ``of_class``
    | ``ElementIsCurveDrivenFilter`` = ``is_curve_driven``
    | ``ElementIsElementTypeFilter`` = ``is_type`` + ``is_not_type``
    | ``ElementOwnerViewFilter`` = ``view``
    | ``ElementLevelFilter`` = ``level`` + ``not_level``
    | ``ElementOwnerViewFilter`` = ``owner_view`` + ``is_view_independent``
    | ``FamilySymbolFilter`` = ``family``
    | ``FamilyInstanceFilter`` = ``symbol``
    | ``ElementParameterFilter`` = ``parameter_filter``
    | ``Exclusion`` = ``exclude``
    | ``UnionWith`` = ``or_collector``
    | ``IntersectWith`` = ``and_collector``
    | ``Custom`` = where

"""

from rpw import revit, DB
from rpw.utils.dotnet import List
from rpw.base import BaseObjectWrapper, BaseObject
from rpw.exceptions import RpwException, RpwTypeError, RpwCoerceError
from rpw.db.element import Element
from rpw.db.builtins import BicEnum, BipEnum
from rpw.ui.selection import Selection
from rpw.db.collection import ElementSet
from rpw.utils.coerce import to_element_id, to_element_ids
from rpw.utils.coerce import to_category, to_class
from rpw.utils.logger import logger
from rpw.utils.logger import deprecate_warning

# More Info on Performance and ElementFilters:
# http://thebuildingcoder.typepad.com/blog/2015/12/quick-slow-and-linq-element-filtering.html


class BaseFilter(BaseObject):
    """ Base Filter and Apply Logic """

    method = 'WherePasses'

    @classmethod
    def process_value(cls, value):
        """
        Filters must implement this method to process the input values and
        convert it into the proper filter or value.

        For example, if the user inputs `level=Level`,
        process value will create a ElementLevelFilter() with the id of Level.

        Additionally, this method can be used for more advanced input
        processing, for example, converting a 'LevelName' into a Level
        to allow for more flexible input options
        """
        raise NotImplemented

    @classmethod
    def apply(cls, doc, collector, value):
        """
        Filters can overide this method to define how the filter is applied
        The default behavious is to chain the ``method`` defined by the filter
        class (ie. WherePasses) to the collector, and feed it the input `value`
        """
        method_name = cls.method
        method = getattr(collector, method_name)

        # FamilyInstanceFilter is the only Filter that  requires Doc
        if cls is not FilterClasses.FamilyInstanceFilter:
            value = cls.process_value(value)
        else:
            value = cls.process_value(value, doc)

        return method(value)


class SuperQuickFilter(BaseFilter):
    """ Preferred Quick """
    priority_group = 0


class QuickFilter(BaseFilter):
    """ Typical Quick """
    priority_group = 1


class SlowFilter(BaseFilter):
    """ Typical Slow """
    priority_group = 2


class SuperSlowFilter(BaseFilter):
    """ Leave it for Last. Must unpack results """
    priority_group = 3

class LogicalFilter(BaseFilter):
    """ Leave it after Last as it must be completed """
    priority_group = 4

class FilterClasses():
    """
    Groups FilterClasses to facilitate discovery.

    # TODO: Move Filter doc to Filter Classes

    Implementation Tracker:
    Quick
        X Revit.DB.ElementCategoryFilter = of_category
        X Revit.DB.ElementClassFilter = of_class
        X Revit.DB.ElementIsCurveDrivenFilter = is_curve_driven
        X Revit.DB.ElementIsElementTypeFilter = is_type / is_not_type
        X Revit.DB.ElementOwnerViewFilter = view
        X Revit.DB.FamilySymbolFilter = family
        X Revit.DB.ExclusionFilter = exclude
        X Revit.DB.IntersectWidth = and_collector
        X Revit.DB.UnionWidth = or_collector
        _ Revit.DB.BoundingBoxContainsPointFilter
        _ Revit.DB.BoundingBoxIntersectsFilter
        _ Revit.DB.BoundingBoxIsInsideFilter
        _ Revit.DB.ElementDesignOptionFilter
        _ Revit.DB.ElementMulticategoryFilter
        _ Revit.DB.ElementMulticlassFilter
        _ Revit.DB.ElementStructuralTypeFilter
        _ Revit.DB.ElementWorksetFilter
        _ Revit.DB.ExtensibleStorage ExtensibleStorageFilter
    Slow
        X Revit.DB.ElementLevelFilter
        X Revit.DB.FamilyInstanceFilter = symbol
        X Revit.DB.ElementParameterFilter
        _ Revit.DB.Architecture RoomFilter
        _ Revit.DB.Architecture RoomTagFilter
        _ Revit.DB.AreaFilter
        _ Revit.DB.AreaTagFilter
        _ Revit.DB.CurveElementFilter
        _ Revit.DB.ElementIntersectsFilter
        _ Revit.DB.ElementPhaseStatusFilter
        _ Revit.DB.Mechanical SpaceFilter
        _ Revit.DB.Mechanical SpaceTagFilter
        _ Revit.DB.PrimaryDesignOptionMemberFilter
        _ Revit.DB.Structure FamilyStructuralMaterialTypeFilter
        _ Revit.DB.Structure StructuralInstanceUsageFilter
        _ Revit.DB.Structure StructuralMaterialTypeFilter
        _ Revit.DB.Structure StructuralWallUsageFilter
        _ Autodesk.Revit.UI.Selection SelectableInViewFilter

    Logical
        _ Revit.DB.LogicalAndFilter = and_filter
        _ Revit.DB.LogicalOrFilter = or_filter

    Others
        X Custom where - uses lambda

    """
    @classmethod
    def get_available_filters(cls):
        """ Discover all Defined Filter Classes """
        filters = []
        for filter_class_name in dir(FilterClasses):
            if filter_class_name.endswith('Filter'):
                filters.append(getattr(FilterClasses, filter_class_name))
        return filters

    @classmethod
    def get_sorted(cls):
        """ Returns Defined Filter Classes sorted by priority """
        return sorted(FilterClasses.get_available_filters(),
                      key=lambda f: f.priority_group)

    class ClassFilter(SuperQuickFilter):
        keyword = 'of_class'

        @classmethod
        def process_value(cls, class_reference):
            class_ = to_class(class_reference)
            return DB.ElementClassFilter(class_)

    class CategoryFilter(SuperQuickFilter):
        keyword = 'of_category'

        @classmethod
        def process_value(cls, category_reference):
            category = to_category(category_reference)
            return DB.ElementCategoryFilter(category)

    class IsTypeFilter(QuickFilter):
        keyword = 'is_type'

        @classmethod
        def process_value(cls, bool_value):
            return DB.ElementIsElementTypeFilter(not(bool_value))

    class IsNotTypeFilter(IsTypeFilter):
        keyword = 'is_not_type'

        @classmethod
        def process_value(cls, bool_value):
            return DB.ElementIsElementTypeFilter(bool_value)

    class FamilySymbolFilter(QuickFilter):
        keyword = 'family'

        @classmethod
        def process_value(cls, family_reference):
            family_id = to_element_id(family_reference)
            return DB.FamilySymbolFilter(family_id)

    class ViewOwnerFilter(QuickFilter):
        keyword = 'owner_view'
        reverse = False

        @classmethod
        def process_value(cls, view_reference):
            if view_reference is not None:
                view_id = to_element_id(view_reference)
            else:
                view_id = DB.ElementId.InvalidElementId
            return DB.ElementOwnerViewFilter(view_id, cls.reverse)

    class ViewIndependentFilter(QuickFilter):
        keyword = 'is_view_independent'

        @classmethod
        def process_value(cls, bool_value):
            view_id = DB.ElementId.InvalidElementId
            return DB.ElementOwnerViewFilter(view_id, not(bool_value))

    class CurveDrivenFilter(QuickFilter):
        keyword = 'is_curve_driven'

        @classmethod
        def process_value(cls, bool_value):
            return DB.ElementIsCurveDrivenFilter(not(bool_value))

    class FamilyInstanceFilter(SlowFilter):
        keyword = 'symbol'

        @classmethod
        def process_value(cls, symbol_reference, doc):
            symbol_id = to_element_id(symbol_reference)
            return DB.FamilyInstanceFilter(doc, symbol_id)

    class LevelFilter(SlowFilter):
        keyword = 'level'
        reverse = False

        @classmethod
        def process_value(cls, level_reference):
            """ Process level= input to allow for level name """
            if isinstance(level_reference, str):
                level = Collector(of_class='Level', is_type=False,
                                  where=lambda x:
                                  x.Name == level_reference)
                try:
                    level_id = level[0].Id
                except IndexError:
                    RpwCoerceError(level_reference, DB.Level)
            else:
                level_id = to_element_id(level_reference)
            return DB.ElementLevelFilter(level_id, cls.reverse)

    class NotLevelFilter(LevelFilter):
        keyword = 'not_level'
        reverse = True

    class ParameterFilter(SlowFilter):
        keyword = 'parameter_filter'

        @classmethod
        def process_value(cls, parameter_filter):
            if isinstance(parameter_filter, ParameterFilter):
                return parameter_filter.unwrap()
            else:
                raise Exception('Shouldnt get here')

    class WhereFilter(SuperSlowFilter):
        """
        Requires Unpacking of each Element. As per the API design,
        this filter must be combined.

        By default, function will test against wrapped elements for easier
        parameter access

        >>> Collector(of_class='FamilyInstance', where=lambda x: 'Desk' in x.name)
        >>> Collector(of_class='Wall', where=lambda x: 'Desk' in x.parameters['Length'] > 5.0)
        """
        keyword = 'where'

        @classmethod
        def apply(cls, doc, collector, func):
            excluded_elements = set()
            for element in collector:
                wrapped_element = Element(element)
                if not func(wrapped_element):
                    excluded_elements.add(element.Id)
            excluded_elements = List[DB.ElementId](excluded_elements)
            if excluded_elements:
                return collector.Excluding(excluded_elements)
            else:
                return collector

    class ExclusionFilter(QuickFilter):
        keyword = 'exclude'

        @classmethod
        def process_value(cls, element_references):
            element_set = ElementSet(element_references)
            return DB.ExclusionFilter(element_set.as_element_id_list)

    class InteresectFilter(LogicalFilter):
        keyword = 'and_collector'

        @classmethod
        def process_value(cls, collector):
            if hasattr(collector, 'unwrap'):
                collector = collector.unwrap()
            return collector

        @classmethod
        def apply(cls, doc, collector, value):
            new_collector = cls.process_value(value)
            return collector.IntersectWith(new_collector)

    class UnionFilter(InteresectFilter):
        keyword = 'or_collector'

        @classmethod
        def apply(cls, doc, collector, value):
            new_collector = cls.process_value(value)
            return collector.UnionWith(new_collector)


class Collector(BaseObjectWrapper):
    """
    Revit FilteredElement Collector Wrapper

    Usage:
        >>> collector = Collector(of_class='View')
        >>> elements = collector.get_elements()

        Multiple Filters:

        >>> Collector(of_class='Wall', is_not_type=True)
        >>> Collector(of_class='ViewSheet', is_not_type=True)
        >>> Collector(of_category='OST_Rooms', level=some_level)
        >>> Collector(symbol=SomeSymbol)
        >>> Collector(owner_view=SomeView)
        >>> Collector(owner_view=None)
        >>> Collector(parameter_filter=parameter_filter)

        Use Enumeration member or its name as a string:

        >>> Collector(of_category='OST_Walls')
        >>> Collector(of_category=DB.BuiltInCategory.OST_Walls)
        >>> Collector(of_class=DB.ViewType)
        >>> Collector(of_class='ViewType')

        Search Document, View, or list of elements

        >>> Collector(of_category='OST_Walls') # doc is default
        >>> Collector(view=SomeView, of_category='OST_Walls') # Doc is default
        >>> Collector(doc=SomeLinkedDoc, of_category='OST_Walls')
        >>> Collector(elements=[Element1, Element2,...], of_category='OST_Walls')
        >>> Collector(owner_view=SomeView)
        >>> Collector(owner_view=None)

    Attributes:
        collector.get_elements(): Returns list of all `collected` elements
        collector.get_first(): Returns first found element, or ``None``
        collector.get_elements(): Returns list with all elements wrapped.
                                    Elements will be instantiated using :any:`Element`

    Wrapped Element:
        self._revit_object = ``Revit.DB.FilteredElementCollector``

    """

    _revit_object_class = DB.FilteredElementCollector

    def __init__(self, **filters):
        """
        Args:
            **filters (``keyword args``): Scope and filters

        Returns:
            Collector (:any:`Collector`): Collector Instance

        Scope Options:
            * ``view`` `(DB.View)`: View Scope (Optional)
            * ``element_ids`` `([ElementId])`: List of Element Ids to limit Collector Scope
            * ``elements`` `([Element])`: List of Elements to limit Collector Scope

        Warning:
            Only one scope filter should be used per query. If more then one is used,
            only one will be applied, in this order ``view`` > ``elements`` > ``element_ids``

        Filter Options:
            * is_type (``bool``): Same as ``WhereElementIsElementType``
            * is_not_type (``bool``): Same as ``WhereElementIsNotElementType``
            * of_class (``Type``): Same as ``OfClass``. Type can be ``DB.SomeType`` or string: ``DB.Wall`` or ``'Wall'``
            * of_category (``BuiltInCategory``): Same as ``OfCategory``. Can be ``DB.BuiltInCategory.OST_Wall`` or ``'Wall'``
            * owner_view (``DB.ElementId, View`): ``WhereElementIsViewIndependent(True)``
            * is_view_independent (``bool``): ``WhereElementIsViewIndependent(True)``
            * family (``DB.ElementId``, ``DB.Element``): Element or ElementId of Family
            * symbol (``DB.ElementId``, ``DB.Element``): Element or ElementId of Symbol
            * level (``DB.Level``, ``DB.ElementId``, ``Level Name``): Level, ElementId of Level, or Level Name
            * not_level (``DB.Level``, ``DB.ElementId``, ``Level Name``): Level, ElementId of Level, or Level Name
            * parameter_filter (:any:`ParameterFilter`): Applies ``ElementParameterFilter``
            * exclude (`element_references`): Element(s) or ElementId(s) to exlude from result
            * and_collector (``collector``): Collector to intersect with. Elements must be present in both
            * or_collector (``collector``): Collector to Union with. Elements must be present on of the two.
            * where (`function`): function to test your elements against

        """
        # Define Filtered Element Collector Scope + Doc
        collector_doc = filters.pop('doc') if 'doc' in filters else revit.doc

        if 'view' in filters:
            view = filters.pop('view')
            view_id = view if isinstance(view, DB.ElementId) else view.Id
            collector = DB.FilteredElementCollector(collector_doc, view_id)
        elif 'elements' in filters:
            elements = filters.pop('elements')
            element_ids = to_element_ids(elements)
            collector = DB.FilteredElementCollector(collector_doc, List[DB.ElementId](element_ids))
        elif 'element_ids' in filters:
            element_ids = filters.pop('element_ids')
            collector = DB.FilteredElementCollector(collector_doc, List[DB.ElementId](element_ids))
        else:
            collector = DB.FilteredElementCollector(collector_doc)

        super(Collector, self).__init__(collector)

        for key in filters.keys():
            if key not in [f.keyword for f in FilterClasses.get_sorted()]:
                raise RpwException('Filter not valid: {}'.format(key))

        self._collector = self._collect(collector_doc, collector, filters)

    def _collect(self, doc, collector, filters):
        """
        Main Internal Recursive Collector Function.

        Args:
            doc (`UI.UIDocument`): Document for the collector.
            collector (`FilteredElementCollector`): FilteredElementCollector
            filters (`dict`): Filters - {'doc': revit.doc, 'of_class': 'Wall'}

        Returns:
            collector (`FilteredElementCollector`): FilteredElementCollector
        """
        for filter_class in FilterClasses.get_sorted():
            if filter_class.keyword not in filters:
                continue
            filter_value = filters.pop(filter_class.keyword)
            logger.debug('Applying Filter: {}:{}'.format(filter_class, filter_value))
            new_collector = filter_class.apply(doc, collector, filter_value)
            return self._collect(doc, new_collector, filters)
        return collector

    def __iter__(self):
        """ Uses iterator to reduce unecessary memory usage """
        # TODO: Depracate or Make return Wrapped ?
        for element in self._collector:
            yield element

    def get_elements(self, wrapped=True):
        """
        Returns list with all elements instantiated using :any:`Element`
        """
        if wrapped:
            return [Element(el) for el in self.__iter__()]
        else:
            return [element for element in self.__iter__()]

    @property
    def elements(self):
        """ Returns list with all elements """
        deprecate_warning('Collector.elements',
                          'Collector.get_elements(wrapped=True)')
        return self.get_elements(wrapped=False)

    @property
    def wrapped_elements(self):
        """ Returns list with all elements instantiated using :any:`Element`"""
        deprecate_warning('Collector.wrapped_elements',
                          'Collector.get_elements(wrapped=True)')
        return self.get_elements(wrapped=True)

    def select(self):
        """ Selects Collector Elements on the UI """
        Selection(self.element_ids)

    def get_first(self, wrapped=True):
        """
        Returns first element or `None`

        Returns:
            Element (`DB.Element`, `None`): First element or None
        """
        try:
            element = self[0]
            return Element(element) if wrapped else element
        except IndexError:
            return None


    # @property
    # def get_first(self):
    #     deprecate_warning('Collector.first', 'Collector.get_first()')
    #     return self.get_first(wrapped=False)

    def get_element_ids(self):
        """
        Returns list with all elements instantiated using :any:`Element`
        """
        return [element_id for element_id in self._collector.ToElementIds()]

    @property
    def element_ids(self):
        deprecate_warning('Collector.element_ids',
                          'Collector.get_element_ids()')
        return self.get_element_ids()

    def __getitem__(self, index):
        # TODO: Depracate or Make return Wrapped ?
        for n, element in enumerate(self.__iter__()):
            if n == index:
                return element
        else:
            raise IndexError('Index {} not in collector {}'.format(index,
                                                                   self))

    def __bool__(self):
        """ Evaluates to `True` if Collector.elements is not empty [] """
        return bool(self.get_elements(wrapped=False))

    def __len__(self):
        """ Returns length of collector.get_elements() """
        try:
            return self._collector.GetElementCount()
        except AttributeError:
            return len(self.get_elements(wrapped=False))  # Revit 2015

    def __repr__(self):
        return super(Collector, self).__repr__(data={'count': len(self)})


class ParameterFilter(BaseObjectWrapper):
    """
    Parameter Filter Wrapper.
    Used to build a parameter filter to be used with the Collector.

    Usage:
        >>> param_id = DB.ElementId(DB.BuiltInParameter.TYPE_NAME)
        >>> parameter_filter = ParameterFilter(param_id, equals='Wall 1')
        >>> collector = Collector(parameter_filter=parameter_filter)

    Returns:
        FilterRule: A filter rule object, depending on arguments.
    """
    _revit_object_class = DB.ElementParameterFilter

    RULES = {
            'equals': 'CreateEqualsRule',
            'not_equals': 'CreateEqualsRule',
            'contains': 'CreateContainsRule',
            'not_contains': 'CreateContainsRule',
            'begins': 'CreateBeginsWithRule',
            'not_begins': 'CreateBeginsWithRule',
            'ends': 'CreateEndsWithRule',
            'not_ends': 'CreateEndsWithRule',
            'greater': 'CreateGreaterRule',
            'not_greater': 'CreateGreaterRule',
            'greater_equal': 'CreateGreaterOrEqualRule',
            'not_greater_equal': 'CreateGreaterOrEqualRule',
            'less': 'CreateLessRule',
            'not_less': 'CreateLessRule',
            'less_equal': 'CreateLessOrEqualRule',
            'not_less_equal': 'CreateLessOrEqualRule',
           }

    CASE_SENSITIVE = True                 # Override with case_sensitive=False
    FLOAT_PRECISION = 0.0013020833333333  # 1/64" in ft:(1/64" = 0.015625)/12

    def __init__(self, parameter_reference, **conditions):
        """
        Creates Parameter Filter Rule

        >>> param_rule = ParameterFilter(param_id, equals=2)
        >>> param_rule = ParameterFilter(param_id, not_equals='a', case_sensitive=True)
        >>> param_rule = ParameterFilter(param_id, not_equals=3, reverse=True)

        Args:
            param_id(DB.ElementID): ElementId of parameter
            **conditions: Filter Rule Conditions and options.

            conditions:
                | ``begins``, ``not_begins``
                | ``contains``, ``not_contains``
                | ``ends``, ``not_ends``
                | ``equals``, ``not_equals``
                | ``less``, ``not_less``
                | ``less_equal``, ``not_less_equal``
                | ``greater``, ``not_greater``
                | ``greater_equal``, ``not_greater_equal``

            options:
                | ``case_sensitive``: Enforces case sensitive, String only
                | ``reverse``: Reverses result of Collector

        """
        parameter_id = self.coerce_param_reference(parameter_reference)
        reverse = conditions.get('reverse', False)
        case_sensitive = conditions.get('case_sensitive', ParameterFilter.CASE_SENSITIVE)
        precision = conditions.get('precision', ParameterFilter.FLOAT_PRECISION)

        for condition in conditions.keys():
            if condition not in ParameterFilter.RULES:
                raise RpwException('Rule not valid: {}'.format(condition))

        rules = []
        for condition_name, condition_value in conditions.iteritems():

            # Returns on of the CreateRule factory method names above
            rule_factory_name = ParameterFilter.RULES.get(condition_name)
            filter_value_rule = getattr(DB.ParameterFilterRuleFactory,
                                        rule_factory_name)

            args = [condition_value]

            if isinstance(condition_value, str):
                args.append(case_sensitive)

            if isinstance(condition_value, float):
                args.append(precision)

            filter_rule = filter_value_rule(parameter_id, *args)
            if 'not_' in condition_name:
                filter_rule = DB.FilterInverseRule(filter_rule)

            logger.debug('ParamFilter Conditions: {}'.format(conditions))
            logger.debug('Case sensitive: {}'.format(case_sensitive))
            logger.debug('Reverse: {}'.format(reverse))
            logger.debug('ARGS: {}'.format(args))
            logger.debug(filter_rule)
            logger.debug(str(dir(filter_rule)))

            rules.append(filter_rule)
        if not rules:
            raise RpwException('malformed filter rule: {}'.format(conditions))

        _revit_object = DB.ElementParameterFilter(List[DB.FilterRule](rules),
                                                  reverse)
        super(ParameterFilter, self).__init__(_revit_object)
        self.conditions = conditions

    def coerce_param_reference(self, parameter_reference):
        if isinstance(parameter_reference, str):
            param_id = BipEnum.get_id(parameter_reference)
        elif isinstance(parameter_reference, DB.ElementId):
            param_id = parameter_reference
        else:
            RpwCoerceError(parameter_reference, ElementId)
        return param_id

    @staticmethod
    def from_element_and_parameter(element, param_name, **conditions):
        """
        Alternative constructor to built Parameter Filter from Element +
        Parameter Name instead of parameter Id

        >>> parameter_filter = ParameterFilter.from_element(element,param_name, less_than=10)
        >>> Collector(parameter_filter=parameter_filter)
        """
        parameter = element.LookupParameter(param_name)
        param_id = parameter.Id
        return ParameterFilter(param_id, **conditions)

    def __repr__(self):
        return super(ParameterFilter, self).__repr__(data=self.conditions)



class UnmanagedFunctionPointerAttribute(Attribute,_Attribute):
 """
 Controls the marshaling behavior of a delegate signature passed as an unmanaged function pointer to or from unmanaged code. This class cannot be inherited.

 

 UnmanagedFunctionPointerAttribute(callingConvention: CallingConvention)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,callingConvention):
  """ __new__(cls: type,callingConvention: CallingConvention) """
  pass
 CallingConvention=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the value of the calling convention.



Get: CallingConvention(self: UnmanagedFunctionPointerAttribute) -> CallingConvention



"""


 BestFitMapping=None
 CharSet=None
 SetLastError=None
 ThrowOnUnmappableChar=None




class TypeConverter(object):
 """
 Provides a unified way of converting types of values to other types,as well as for accessing standard values and subproperties.

 

 TypeConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: TypeConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Returns whether this converter can convert an object of the given type to the type of this 

    converter,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   sourceType: A System.Type that represents the type you want to convert from.

   Returns: true if this converter can perform the conversion; otherwise,false.

  CanConvertFrom(self: TypeConverter,sourceType: Type) -> bool

  

   Returns whether this converter can convert an object of the given type to the type of this 

    converter.

  

  

   sourceType: A System.Type that represents the type you want to convert from.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: TypeConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Returns whether this converter can convert the object to the specified type,using the specified 

    context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that represents the type you want to convert to.

   Returns: true if this converter can perform the conversion; otherwise,false.

  CanConvertTo(self: TypeConverter,destinationType: Type) -> bool

  

   Returns whether this converter can convert the object to the specified type.

  

   destinationType: A System.Type that represents the type you want to convert to.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: TypeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the given object to the type of this converter,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: The System.Globalization.CultureInfo to use as the current culture.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.

  ConvertFrom(self: TypeConverter,value: object) -> object

  

   Converts the given value to the type of this converter.

  

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertFromInvariantString(self,*__args):
  """
  ConvertFromInvariantString(self: TypeConverter,context: ITypeDescriptorContext,text: str) -> object

  

   Converts the given string to the type of this converter,using the invariant culture and the 

    specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   text: The System.String to convert.

   Returns: An System.Object that represents the converted text.

  ConvertFromInvariantString(self: TypeConverter,text: str) -> object

  

   Converts the given string to the type of this converter,using the invariant culture.

  

   text: The System.String to convert.

   Returns: An System.Object that represents the converted text.
  """
  pass
 def ConvertFromString(self,*__args):
  """
  ConvertFromString(self: TypeConverter,context: ITypeDescriptorContext,culture: CultureInfo,text: str) -> object

  

   Converts the given text to an object,using the specified context and culture information.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo. If null is passed,the current culture is assumed.

   text: The System.String to convert.

   Returns: An System.Object that represents the converted text.

  ConvertFromString(self: TypeConverter,context: ITypeDescriptorContext,text: str) -> object

  

   Converts the given text to an object,using the specified context.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   text: The System.String to convert.

   Returns: An System.Object that represents the converted text.

  ConvertFromString(self: TypeConverter,text: str) -> object

  

   Converts the specified text to an object.

  

   text: The text representation of the object to convert.

   Returns: An System.Object that represents the converted text.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: TypeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the given value object to the specified type,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo. If null is passed,the current culture is assumed.

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value parameter to.

   Returns: An System.Object that represents the converted value.

  ConvertTo(self: TypeConverter,value: object,destinationType: Type) -> object

  

   Converts the given value object to the specified type,using the arguments.

  

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value parameter to.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertToInvariantString(self,*__args):
  """
  ConvertToInvariantString(self: TypeConverter,context: ITypeDescriptorContext,value: object) -> str

  

   Converts the specified value to a culture-invariant string representation,using the specified 

    context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: The System.Object to convert.

   Returns: A System.String that represents the converted value.

  ConvertToInvariantString(self: TypeConverter,value: object) -> str

  

   Converts the specified value to a culture-invariant string representation.

  

   value: The System.Object to convert.

   Returns: A System.String that represents the converted value.
  """
  pass
 def ConvertToString(self,*__args):
  """
  ConvertToString(self: TypeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> str

  

   Converts the given value to a string representation,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: A System.Globalization.CultureInfo. If null is passed,the current culture is assumed.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.

  ConvertToString(self: TypeConverter,context: ITypeDescriptorContext,value: object) -> str

  

   Converts the given value to a string representation,using the given context.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.

  ConvertToString(self: TypeConverter,value: object) -> str

  

   Converts the specified value to a string representation.

  

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def CreateInstance(self,*__args):
  """
  CreateInstance(self: TypeConverter,context: ITypeDescriptorContext,propertyValues: IDictionary) -> object

  

   Creates an instance of the type that this System.ComponentModel.TypeConverter is associated 

    with,using the specified context,given a set of property values for the object.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   propertyValues: An System.Collections.IDictionary of new property values.

   Returns: An System.Object representing the given System.Collections.IDictionary,or null if the object 

    cannot be created. This method always returns null.

  

  CreateInstance(self: TypeConverter,propertyValues: IDictionary) -> object

  

   Re-creates an System.Object given a set of property values for the object.

  

   propertyValues: An System.Collections.IDictionary that represents a dictionary of new property values.

   Returns: An System.Object representing the given System.Collections.IDictionary,or null if the object 

    cannot be created. This method always returns null.
  """
  pass
 def GetConvertFromException(self,*args):
  """
  GetConvertFromException(self: TypeConverter,value: object) -> Exception

  

   Returns an exception to throw when a conversion cannot be performed.

  

   value: The System.Object to convert,or null if the object is not available.

   Returns: An System.Exception that represents the exception to throw when a conversion cannot be performed.
  """
  pass
 def GetConvertToException(self,*args):
  """
  GetConvertToException(self: TypeConverter,value: object,destinationType: Type) -> Exception

  

   Returns an exception to throw when a conversion cannot be performed.

  

   value: The System.Object to convert,or null if the object is not available.

   destinationType: A System.Type that represents the type the conversion was trying to convert to.

   Returns: An System.Exception that represents the exception to throw when a conversion cannot be performed.
  """
  pass
 def GetCreateInstanceSupported(self,context=None):
  """
  GetCreateInstanceSupported(self: TypeConverter,context: ITypeDescriptorContext) -> bool

  

   Returns whether changing a value on this object requires a call to 

    System.ComponentModel.TypeConverter.CreateInstance(System.Collections.IDictionary) to create a 

    new value,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if changing a property on this object requires a call to 

    System.ComponentModel.TypeConverter.CreateInstance(System.Collections.IDictionary) to create a 

    new value; otherwise,false.

  

  GetCreateInstanceSupported(self: TypeConverter) -> bool

  

   Returns whether changing a value on this object requires a call to the 

    System.ComponentModel.TypeConverter.CreateInstance(System.Collections.IDictionary) method to 

    create a new value.

  

   Returns: true if changing a property on this object requires a call to 

    System.ComponentModel.TypeConverter.CreateInstance(System.Collections.IDictionary) to create a 

    new value; otherwise,false.
  """
  pass
 def GetProperties(self,*__args):
  """
  GetProperties(self: TypeConverter,context: ITypeDescriptorContext,value: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Returns a collection of properties for the type of array specified by the value parameter,using 

    the specified context and attributes.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: An System.Object that specifies the type of array for which to get properties.

   attributes: An array of type System.Attribute that is used as a filter.

   Returns: A System.ComponentModel.PropertyDescriptorCollection with the properties that are exposed for 

    this data type,or null if there are no properties.

  

  GetProperties(self: TypeConverter,context: ITypeDescriptorContext,value: object) -> PropertyDescriptorCollection

  

   Returns a collection of properties for the type of array specified by the value parameter,using 

    the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: An System.Object that specifies the type of array for which to get properties.

   Returns: A System.ComponentModel.PropertyDescriptorCollection with the properties that are exposed for 

    this data type,or null if there are no properties.

  

  GetProperties(self: TypeConverter,value: object) -> PropertyDescriptorCollection

  

   Returns a collection of properties for the type of array specified by the value parameter.

  

   value: An System.Object that specifies the type of array for which to get properties.

   Returns: A System.ComponentModel.PropertyDescriptorCollection with the properties that are exposed for 

    this data type,or null if there are no properties.
  """
  pass
 def GetPropertiesSupported(self,context=None):
  """
  GetPropertiesSupported(self: TypeConverter,context: ITypeDescriptorContext) -> bool

  

   Returns whether this object supports properties,using the specified context.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if System.ComponentModel.TypeConverter.GetProperties(System.Object) should be called to 

    find the properties of this object; otherwise,false.

  

  GetPropertiesSupported(self: TypeConverter) -> bool

  

   Returns whether this object supports properties.

   Returns: true if System.ComponentModel.TypeConverter.GetProperties(System.Object) should be called to 

    find the properties of this object; otherwise,false.
  """
  pass
 def GetStandardValues(self,context=None):
  """
  GetStandardValues(self: TypeConverter,context: ITypeDescriptorContext) -> StandardValuesCollection

  

   Returns a collection of standard values for the data type this type converter is designed for 

    when provided with a format context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context that can be used 

    to extract additional information about the environment from which this converter is invoked. 

    This parameter or properties of this parameter can be null.

  

   Returns: A System.ComponentModel.TypeConverter.StandardValuesCollection that holds a standard set of 

    valid values,or null if the data type does not support a standard set of values.

  

  GetStandardValues(self: TypeConverter) -> ICollection

  

   Returns a collection of standard values from the default context for the data type this type 

    converter is designed for.

  

   Returns: A System.ComponentModel.TypeConverter.StandardValuesCollection containing a standard set of 

    valid values,or null if the data type does not support a standard set of values.
  """
  pass
 def GetStandardValuesExclusive(self,context=None):
  """
  GetStandardValuesExclusive(self: TypeConverter,context: ITypeDescriptorContext) -> bool

  

   Returns whether the collection of standard values returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exclusive list of possible values,

    using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if the System.ComponentModel.TypeConverter.StandardValuesCollection returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exhaustive list of possible values; 

    false if other values are possible.

  

  GetStandardValuesExclusive(self: TypeConverter) -> bool

  

   Returns whether the collection of standard values returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exclusive list.

  

   Returns: true if the System.ComponentModel.TypeConverter.StandardValuesCollection returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exhaustive list of possible values; 

    false if other values are possible.
  """
  pass
 def GetStandardValuesSupported(self,context=None):
  """
  GetStandardValuesSupported(self: TypeConverter,context: ITypeDescriptorContext) -> bool

  

   Returns whether this object supports a standard set of values that can be picked from a list,

    using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if System.ComponentModel.TypeConverter.GetStandardValues should be called to find a common 

    set of values the object supports; otherwise,false.

  

  GetStandardValuesSupported(self: TypeConverter) -> bool

  

   Returns whether this object supports a standard set of values that can be picked from a list.

   Returns: true if System.ComponentModel.TypeConverter.GetStandardValues should be called to find a common 

    set of values the object supports; otherwise,false.
  """
  pass
 def IsValid(self,*__args):
  """
  IsValid(self: TypeConverter,context: ITypeDescriptorContext,value: object) -> bool

  

   Returns whether the given value object is valid for this type and for the specified context.

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: The System.Object to test for validity.

   Returns: true if the specified value is valid for this object; otherwise,false.

  IsValid(self: TypeConverter,value: object) -> bool

  

   Returns whether the given value object is valid for this type.

  

   value: The object to test for validity.

   Returns: true if the specified value is valid for this object; otherwise,false.
  """
  pass
 def SortProperties(self,*args):
  """
  SortProperties(self: TypeConverter,props: PropertyDescriptorCollection,names: Array[str]) -> PropertyDescriptorCollection

  

   Sorts a collection of properties.

  

   props: A System.ComponentModel.PropertyDescriptorCollection that has the properties to sort.

   names: An array of names in the order you want the properties to appear in the collection.

   Returns: A System.ComponentModel.PropertyDescriptorCollection that contains the sorted properties.
  """
  pass
 SimplePropertyDescriptor=None
 StandardValuesCollection=None




class IFeatureSupport:
 """ Specifies a standard interface for retrieving feature information from the current system. """
 def GetVersionPresent(self,feature):
  """
  GetVersionPresent(self: IFeatureSupport,feature: object) -> Version

  

   Retrieves the version of the specified feature.

  

   feature: The feature whose version is requested.

   Returns: A System.Version representing the version number of the specified feature; or null if the 

    feature is not installed.
  """
  pass
 def IsPresent(self,feature,minimumVersion=None):
  """
  IsPresent(self: IFeatureSupport,feature: object,minimumVersion: Version) -> bool

  

   Determines whether the specified or newer version of the specified feature is currently 

    available on the system.

  

  

   feature: The feature to look for.

   minimumVersion: A System.Version representing the minimum version number of the feature to look for.

   Returns: true if the requested version of the feature is present; otherwise,false.

  IsPresent(self: IFeatureSupport,feature: object) -> bool

  

   Determines whether any version of the specified feature is currently available on the system.

  

   feature: The feature to look for.

   Returns: true if the feature is present; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class ITypeDescriptorContext(IServiceProvider):
 """ Provides contextual information about a component,such as its container and property descriptor. """
 def OnComponentChanged(self):
  """
  OnComponentChanged(self: ITypeDescriptorContext)

   Raises the System.ComponentModel.Design.IComponentChangeService.ComponentChanged event.
  """
  pass
 def OnComponentChanging(self):
  """
  OnComponentChanging(self: ITypeDescriptorContext) -> bool

  

   Raises the System.ComponentModel.Design.IComponentChangeService.ComponentChanging event.

   Returns: true if this object can be changed; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 Container=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the container representing this System.ComponentModel.TypeDescriptor request.



Get: Container(self: ITypeDescriptorContext) -> IContainer



"""

 Instance=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the object that is connected with this type descriptor request.



Get: Instance(self: ITypeDescriptorContext) -> object



"""

 PropertyDescriptor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.ComponentModel.PropertyDescriptor that is associated with the given context item.



Get: PropertyDescriptor(self: ITypeDescriptorContext) -> PropertyDescriptor



"""





# encoding: utf-8
# module System.Windows.Input calls itself Input
# from System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class ICommand:
 # no doc
 def CanExecute(self,parameter):
  """ CanExecute(self: ICommand,parameter: object) -> bool """
  pass
 def Execute(self,parameter):
  """ Execute(self: ICommand,parameter: object) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 CanExecuteChanged=None





class ItemContainerGenerator(object,IRecyclingItemContainerGenerator,IItemContainerGenerator,IWeakEventListener):
 """ Generates the user interface (UI) on behalf of its host,such as an�System.Windows.Controls.ItemsControl. """
 def ContainerFromIndex(self,index):
  """
  ContainerFromIndex(self: ItemContainerGenerator,index: int) -> DependencyObject

  

   Returns the element corresponding to the item at the given index within the 

    System.Windows.Controls.ItemCollection.

  

  

   index: The index of the desired item.

   Returns: Returns the element corresponding to the item at the given index within the 

    System.Windows.Controls.ItemCollection or returns null if the item is not realized.
  """
  pass
 def ContainerFromItem(self,item):
  """
  ContainerFromItem(self: ItemContainerGenerator,item: object) -> DependencyObject

  

   Returns the System.Windows.UIElement corresponding to the given item.

  

   item: The System.Object item to find the System.Windows.UIElement for.

   Returns: A System.Windows.UIElement that corresponds to the given item. Returns null if the item does not 

    belong to the item collection,or if a System.Windows.UIElement has not been generated for it.
  """
  pass
 def GenerateBatches(self):
  """ GenerateBatches(self: ItemContainerGenerator) -> IDisposable """
  pass
 def IndexFromContainer(self,container,returnLocalIndex=None):
  """
  IndexFromContainer(self: ItemContainerGenerator,container: DependencyObject,returnLocalIndex: bool) -> int

  IndexFromContainer(self: ItemContainerGenerator,container: DependencyObject) -> int

  

   Returns the index to an item that corresponds to the specified,generated 

    System.Windows.UIElement.

  

  

   container: The System.Windows.DependencyObject that corresponds to the item to the index to be returned.

   Returns: An System.Int32 index to an item that corresponds to the specified,generated 

    System.Windows.UIElement or -1 if container is not found.
  """
  pass
 def ItemFromContainer(self,container):
  """
  ItemFromContainer(self: ItemContainerGenerator,container: DependencyObject) -> object

  

   Returns the item that corresponds to the specified,generated System.Windows.UIElement.

  

   container: The System.Windows.DependencyObject that corresponds to the item to be returned.

   Returns: A System.Windows.DependencyObject that is the item which corresponds to the specified,generated 

    System.Windows.UIElement. If the System.Windows.UIElement has not been generated,

    System.Windows.DependencyProperty.UnsetValue is returned.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Items=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: Items(self: ItemContainerGenerator) -> ReadOnlyCollection[object]



"""

 Status=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """The generation status of the System.Windows.Controls.ItemContainerGenerator.



Get: Status(self: ItemContainerGenerator) -> GeneratorStatus



"""


 ItemsChanged=None
 StatusChanged=None




"""UI functions."""
# pylint: disable=import-error,invalid-name,broad-except

import os.path as op
from pyrevit import HOST_APP
from pyrevit.framework import clr
from pyrevit.framework import IntPtr
from pyrevit.framework import Interop, Windows
from pyrevit.api import AdInternal as ai
from pyrevit.api import AdPrivate as ap
from pyrevit.api import AdWindows as ad
from pyrevit.api import UIFramework as uf
from pyrevit.api import UIFrameworkServices as ufs
from pyrevit.labs import Common
from pyrevit.runtime import types
from pyrevit.coreutils import envvars
from pyrevit.extensions import ICON_FILE_FORMAT, ICON_DARK_SUFFIX
from Autodesk.Revit.UI import UIThemeManager, UITheme


def get_mainwindow_hwnd():
    """Get the handle of the main window.

    Returns:
        (intptr): The handle of the main window.
    """
    return HOST_APP.proc_window


def get_mainwindow():
    """Get the main window of the application.

    Returns:
        (MainWindow): The root visual of the main window.
    """
    try:
        hwnd_source = Interop.HwndSource.FromHwnd(HOST_APP.proc_window)
        return hwnd_source.RootVisual
    except Exception:
        pass


def get_statusbar_hwnd():
    """Retrieves the handle of the status bar control belonging to the main window.

    Returns:
        (IntPtr): The handle of the status bar control.
    """
    return Common.User32.FindWindowEx(
        get_mainwindow_hwnd(),
        IntPtr.Zero,
        "msctls_statusbar32",
        "")


def set_statusbar_text(text):
    """Sets the text of the status bar.

    Parameters:
        text (str): The text to be displayed in the status bar.

    Returns:
        (bool): True if the text was successfully set, False otherwise.
    """
    status_bar_ptr = get_statusbar_hwnd()

    if status_bar_ptr != IntPtr.Zero:
        Common.User32.SetWindowText(status_bar_ptr, text)
        return True

    return False


def get_window_rectangle():
    """Get the rectangle coordinates of the main window.

    Returns:
        (Tuple[int, int, int, int]): The left, top, right, and bottom 
            coordinates of the window rectangle.
    """
    return Common.User32.GetWindowRect(get_mainwindow_hwnd())


def is_infocenter_visible():
    """Check if the InfoCenter toolbar is visible.

    Returns:
        (bool): True if the InfoCenter toolbar is visible, False otherwise.
    """
    return ad.ComponentManager.InfoCenterToolBar.Visibility == \
        Windows.Visibility.Visible


def toggle_infocenter():
    """Toggles the visibility of the InfoCenter toolbar.

    This function retrieves the current visibility state of the InfoCenter 
    toolbar and toggles it to the opposite state.
    If the toolbar is currently collapsed, it will be set to visible,
    and if it is currently visible, it will be set to collapsed.
    The function then returns the visibility state of the InfoCenter toolbar
    after the toggle operation.

    Returns:
        (bool): True if the InfoCenter toolbar is visible, False otherwise.
    """
    current_state = ad.ComponentManager.InfoCenterToolBar.Visibility
    is_hidden = (current_state == Windows.Visibility.Collapsed)
    ad.ComponentManager.InfoCenterToolBar.Visibility = \
        Windows.Visibility.Visible if is_hidden else \
            Windows.Visibility.Collapsed
    return is_infocenter_visible()


def get_ribbon_roottype():
    """Get the type of the ribbon root.

    Returns:
        (type): type of the ribbon root
    """
    ap_assm = clr.GetClrType(ap.Windows.RibbonTabList).Assembly
    for apt in ap_assm.GetTypes():
        if 'PanelSetListView' in apt.Name:
            return apt


def get_current_theme():
    """Get the current UI theme.

    Returns:
        UITheme (UITheme): The current UI theme.
    """
    return UIThemeManager.CurrentTheme


def set_current_theme(theme='Dark'):
    """Sets the current UI theme to either 'Dark' or 'Light'.

    Args:
        theme (str, optional): The theme to set. Defaults to 'Dark'.
    """
    if theme == 'Dark':
        UIThemeManager.CurrentTheme = UITheme.Dark
    else:
        UIThemeManager.CurrentTheme = UITheme.Light


def resolve_icon_file(directory, icon_name):
    """Resolves the icon file path based on the current UI theme.

    Args:
        directory (str): The directory where the icon file is located.
        icon_name (str): The name of the icon file.

    Returns:
        full_file_path (str): The full file path of the icon file.
    """
    full_file_path = op.join(directory, icon_name)

    if HOST_APP.is_newer_than(2024, True) and get_current_theme() == UITheme.Dark:
        dark_icon_name = op.splitext(icon_name)[0] + ICON_DARK_SUFFIX + ICON_FILE_FORMAT
        dark_file_path = op.join(directory, dark_icon_name)
        full_file_path = dark_file_path if op.exists(dark_file_path) else full_file_path

    return full_file_path if op.exists(full_file_path) else None



class GenericSecurityDescriptor(object):
 """ Represents a security descriptor. A security descriptor includes an owner,a primary group,a Discretionary Access Control List (DACL),and a System Access Control List (SACL). """
 def GetBinaryForm(self,binaryForm,offset):
  """
  GetBinaryForm(self: GenericSecurityDescriptor,binaryForm: Array[Byte],offset: int)

   Returns an array of byte values that represents the information contained in this 

    System.Security.AccessControl.GenericSecurityDescriptor object.

  

  

   binaryForm: The byte array into which the contents of the 

    System.Security.AccessControl.GenericSecurityDescriptor is marshaled.

  

   offset: The offset at which to start marshaling.
  """
  pass
 def GetSddlForm(self,includeSections):
  """
  GetSddlForm(self: GenericSecurityDescriptor,includeSections: AccessControlSections) -> str

  

   Returns the Security Descriptor Definition Language (SDDL) representation of the specified 

    sections of the security descriptor that this 

    System.Security.AccessControl.GenericSecurityDescriptor object represents.

  

  

   includeSections: Specifies which sections (access rules,audit rules,primary group,owner) of the security 

    descriptor to get.

  

   Returns: The SDDL representation of the specified sections of the security descriptor associated with 

    this System.Security.AccessControl.GenericSecurityDescriptor object.
  """
  pass
 @staticmethod
 def IsSddlConversionSupported():
  """
  IsSddlConversionSupported() -> bool

  

   Returns a boolean value that specifies whether the security descriptor associated with this  

    System.Security.AccessControl.GenericSecurityDescriptor object can be converted to the Security 

    Descriptor Definition Language (SDDL) format.

  

   Returns: true if the security descriptor associated with this  

    System.Security.AccessControl.GenericSecurityDescriptor object can be converted to the Security 

    Descriptor Definition Language (SDDL) format; otherwise,false.
  """
  pass
 BinaryLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the length,in bytes,of the binary representation of the current System.Security.AccessControl.GenericSecurityDescriptor object. This length should be used before marshaling the ACL into a binary array with the System.Security.AccessControl.GenericSecurityDescriptor.GetBinaryForm method.



Get: BinaryLength(self: GenericSecurityDescriptor) -> int



"""

 ControlFlags=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets values that specify behavior of the System.Security.AccessControl.GenericSecurityDescriptor object.



Get: ControlFlags(self: GenericSecurityDescriptor) -> ControlFlags



"""

 Group=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the primary group for this System.Security.AccessControl.GenericSecurityDescriptor object.



Get: Group(self: GenericSecurityDescriptor) -> SecurityIdentifier



Set: Group(self: GenericSecurityDescriptor)=value

"""

 Owner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the owner of the object associated with this System.Security.AccessControl.GenericSecurityDescriptor object.



Get: Owner(self: GenericSecurityDescriptor) -> SecurityIdentifier



Set: Owner(self: GenericSecurityDescriptor)=value

"""


 Revision=None




"""Elements selection utilities."""
from pyrevit import HOST_APP, DOCS, PyRevitException
from pyrevit import framework, DB, UI
from pyrevit.coreutils.logger import get_logger

from pyrevit.revit import ensure
from pyrevit.revit import query
from Autodesk.Revit import Exceptions as RevitExceptions


__all__ = ('pick_element', 'pick_element_by_category',
           'pick_elements', 'pick_elements_by_category',
           'get_picked_elements', 'get_picked_elements_by_category',
           'pick_edge', 'pick_edges',
           'pick_face', 'pick_faces',
           'pick_linked', 'pick_linkeds',
           'pick_elementpoint', 'pick_elementpoints',
           'pick_point', 'pick_rectangle', 'get_selection_category_set',
           'get_selection')


# pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


class ElementSelection:
    """Element selection handler.

    Args:
        element_list (list[DB.Element]): list of selected elements
    """
    def __init__(self, element_list=None):
        if element_list is None:
            if HOST_APP.uidoc:
                self._refs = \
                    [x for x in HOST_APP.uidoc.Selection.GetElementIds()]
            else:
                self._refs = []
        else:
            self._refs = ElementSelection.get_element_ids(element_list)

    def __len__(self):
        return len(self._refs)

    def __iter__(self):
        for elref in self._refs:
            yield DOCS.doc.GetElement(elref)

    def __getitem__(self, index):
        return self.elements[index]

    def __contains__(self, item):
        if isinstance(item, DB.Element):
            elref = item.Id
        elif isinstance(item, DB.ElementId):
            elref = item
        else:
            elref = DB.ElementId.InvalidElementId
        return elref in self._refs

    @classmethod
    def get_element_ids(cls, mixed_list):
        return ensure.ensure_element_ids(mixed_list)

    @property
    def is_empty(self):
        return len(self._refs) == 0

    @property
    def elements(self):
        return [DOCS.doc.GetElement(x) for x in self._refs]

    @property
    def element_ids(self):
        return self._refs

    @property
    def first(self):
        if self._refs:
            return DOCS.doc.GetElement(self._refs[0])

    @property
    def last(self):
        if self._refs:
            return DOCS.doc.GetElement(self._refs[-1])

    def set_to(self, element_list):
        self._refs = ElementSelection.get_element_ids(element_list)
        HOST_APP.uidoc.Selection.SetElementIds(
            framework.List[DB.ElementId](self._refs)
            )
        HOST_APP.uidoc.RefreshActiveView()

    def clear(self):
        HOST_APP.uidoc.Selection.SetElementIds(
            framework.List[DB.ElementId]([DB.ElementId.InvalidElementId])
        )
        HOST_APP.uidoc.RefreshActiveView()

    def append(self, element_list):
        self._refs.extend(ElementSelection.get_element_ids(element_list))
        self.set_to(self._refs)

    def include(self, element_type):
        refs = [x for x in self._refs
                if isinstance(DOCS.doc.GetElement(x),
                              element_type)]
        return ElementSelection(refs)

    def exclude(self, element_type):
        refs = [x for x in self._refs
                if not isinstance(DOCS.doc.GetElement(x),
                                  element_type)]
        return ElementSelection(refs)

    def no_views(self):
        return self.exclude(DB.View)

    def only_views(self):
        return self.include(DB.View)

    def expand_groups(self):
        expanded_refs = []
        for element in self.elements:
            if isinstance(element, DB.Group):
                expanded_refs.extend(element.GetMemberIds())
            else:
                expanded_refs.append(element.Id)
        self._refs = expanded_refs


class PickByCategorySelectionFilter(UI.Selection.ISelectionFilter):
    def __init__(self, category_id):
        self.category_id = category_id

    # standard API override function
    def AllowElement(self, element):
        if element.Category and self.category_id == element.Category.Id:
            return True
        else:
            return False

    # standard API override function
    def AllowReference(self, refer, point):  # pylint: disable=W0613
        return False


def _pick_obj(obj_type, message, multiple=False, world=False, selection_filter=None):
    mlogger.debug(
        "Picking elements: %s " "message: %s " "multiple: %s " "world: %s",
        obj_type,
        message,
        multiple,
        world,
    )
    picker_func = (
        HOST_APP.uidoc.Selection.PickObjects
        if multiple
        else HOST_APP.uidoc.Selection.PickObject
    )
    try:
        pick_result = (
            picker_func(obj_type, selection_filter, message)
            if selection_filter
            else picker_func(obj_type, message)
        )
    except RevitExceptions.OperationCanceledException:
        mlogger.debug("Operation canceled by user")
        return None
    refs = list(pick_result) if multiple else [pick_result]
    if not refs:
        mlogger.debug("Nothing picked by user")
        return None

    mlogger.debug("Picked elements are: %s", refs)

    if obj_type == UI.Selection.ObjectType.Element:
        return_values = [DOCS.doc.GetElement(ref) for ref in refs]
    elif obj_type == UI.Selection.ObjectType.PointOnElement:
        if world:
            return_values = [ref.GlobalPoint for ref in refs]
        else:
            return_values = [ref.UVPoint for ref in refs]
    else:
        return_values = [
            DOCS.doc.GetElement(ref).GetGeometryObjectFromReference(ref) for ref in refs
        ]

    mlogger.debug("Processed return elements are: %s", return_values)

    if len(return_values) > 1 or multiple:
        return return_values
    if len(return_values) == 1:
        return return_values[0]
    mlogger.error("Error processing picked elements. return_values should be a list.")


def pick_element(message=''):
    """Asks the user to pick an element.

    Args:
        message (str): An optional message to display.

    Returns:
        (Element): element selected by the user.
    """
    return _pick_obj(UI.Selection.ObjectType.Element,
                     message)


def pick_element_by_category(cat_name_or_builtin, message=''):
    """Returns the element of the specified category picked by the user.

    Args:
        cat_name_or_builtin (str): name or built-in category of the element
            to pick.
        message (str, optional): message to display on selection.
            Defaults to ''.

    Returns:
        (Element): picked element.

    Raises:
        PyRevitException: If no category matches the specified name or builtin.
    """
    category = query.get_category(cat_name_or_builtin)
    if category:
        pick_filter = PickByCategorySelectionFilter(category.Id)
        return _pick_obj(UI.Selection.ObjectType.Element,
                         message,
                         selection_filter=pick_filter)
    else:
        raise PyRevitException("Can not determine category id from: {}"
                               .format(cat_name_or_builtin))


def pick_elementpoint(message='', world=False):
    """Returns the element point selected by the user.

    Args:
        message (str, optional): message to display. Defaults to ''.
        world (bool, optional): whether to use world coordinates. Defaults to False.

    Returns:
        (PointOnElement): The selected point.
    """
    return _pick_obj(UI.Selection.ObjectType.PointOnElement,
                     message,
                     world=world)


def pick_edge(message=''):
    """Returns the edge selected by the user.

    Args:
        message (str, optional): message to display. Defaults to ''.

    Returns:
        (Edge): The selected edge.
    """
    return _pick_obj(UI.Selection.ObjectType.Edge,
                     message)


def pick_face(message=''):
    """Returns the face selected by the user.

    Args:
        message (str, optional): message to display. Defaults to ''.

    Returns:
        (Face): The selected face.
    """
    return _pick_obj(UI.Selection.ObjectType.Face,
                     message)


def pick_linked(message=''):
    """Returns the linked element selected by the user.

    Args:
        message (str, optional): message to display. Defaults to ''.

    Returns:
        (LinkedElement): The selected linked element.
    """
    return _pick_obj(UI.Selection.ObjectType.LinkedElement,
                     message)


def pick_elements(message=''):
    """Asks the user to pick multiple elements.

    Args:
        message (str): An optional message to display.

    Returns:
        (list[Element]): elements selected by the user.
    """
    return _pick_obj(UI.Selection.ObjectType.Element,
                     message,
                     multiple=True)


def pick_elements_by_category(cat_name_or_builtin, message=''):
    """Returns the elements of the specified category picked by the user.

    Args:
        cat_name_or_builtin (str): name or built-in category of the elements
            to pick.
        message (str, optional): message to display on selection.
            Defaults to ''.

    Returns:
        (list[Element]): picked elements.

    Raises:
        PyRevitException: If no category matches the specified name or builtin.
    """
    category = query.get_category(cat_name_or_builtin)
    if category:
        pick_filter = PickByCategorySelectionFilter(category.Id)
        return _pick_obj(UI.Selection.ObjectType.Element,
                         message,
                         multiple=True,
                         selection_filter=pick_filter)
    else:
        raise PyRevitException("Can not determine category id from: {}"
                               .format(cat_name_or_builtin))


def get_picked_elements(message=''):
    """Allows the user to pick multple elements, one at a time.

    It keeps asking the user to pick an element until no elements are selected.

    Args:
        message (str, optional): The message to display. Defaults to ''.

    Yields:
        (DB.Element): selected element
    """
    picked_element = True
    while picked_element:
        picked_element = pick_element(message=message)
        if not picked_element:
            break
        yield picked_element


def get_picked_elements_by_category(cat_name_or_builtin, message=''):
    """Pick elements by category.

    Keeps asking the user to pick an element until no elements are selected.

    Args:
        cat_name_or_builtin (str): category name or built-in category.
        message (str, optional): message to display while picking elements.

    Yields:
        (DB.Element): The picked elements from the specified category.
    """
    picked_element = True
    while picked_element:
        picked_element = pick_element_by_category(cat_name_or_builtin,
                                                  message=message)
        if not picked_element:
            break
        yield picked_element


def pick_elementpoints(message='', world=False):
    """Selects element points.

    Args:
        message (str): The message to display when selecting element points.
        world (bool, optional): Select points in world coordinates. Defaults to False.

    Returns:
        (list[PointOnElement]): selected element points.
    """
    return _pick_obj(UI.Selection.ObjectType.PointOnElement,
                     message,
                     multiple=True, world=world)


def pick_edges(message=''):
    """Selects edges.

    Args:
        message (str): The message to display when selecting edges.

    Returns:
        (list[Edge]): selected edges.
    """
    return _pick_obj(UI.Selection.ObjectType.Edge,
                     message,
                     multiple=True)


def pick_faces(message=''):
    """Selects faces.

    Args:
        message (str): The message to display when selecting the faces.

    Returns:
        (list[Face]): selected faces.
    """
    return _pick_obj(UI.Selection.ObjectType.Face,
                     message,
                     multiple=True)


def pick_linkeds(message=''):
    """Selects linked elements.

    Args:
        message (str): The message to display when selecting linked elements.

    Returns:
        (list[LinkedElement]): selected linked elements.
    """
    return _pick_obj(UI.Selection.ObjectType.LinkedElement,
                     message,
                     multiple=True)


def pick_point(message=''):
    """Pick a point from the user interface.

    Args:
        message (str): A message to display when prompting for the point.

    Returns:
        (tuple or None): A tuple representing the picked point as (x, y, z)
            coordinates, or None if no point was picked or an error occurred.
    """
    try:
        return HOST_APP.uidoc.Selection.PickPoint(message)
    except Exception:
        return None


def pick_rectangle(message='', pick_filter=None):
    """Picks elements from the user interface by specifying a rectangular area.

    Args:
        message (str, optional): A custom message to display when prompting
            the user to pick elements. Default is an empty string.
        pick_filter (object, optional): An object specifying the filter to apply
            when picking elements. Default is None.

    Returns:
        (list[DB.ElementId]): The selected elements.
    """
    if pick_filter:
        return HOST_APP.uidoc.Selection.PickElementsByRectangle(pick_filter,
                                                                message)
    else:
        return HOST_APP.uidoc.Selection.PickElementsByRectangle(message)


def get_selection_category_set():
    """Returns a CategorySet with the categories of the selected elements.

    Returns:
        (CategorySet): categories of the selected elements.
    """
    selection = ElementSelection()
    cset = DB.CategorySet()
    for element in selection:
        cset.Insert(element.Category)
    return cset


def get_selection():
    """Returns the current selected items.

    Returns:
        (ElementSelection): the current selected items
    """
    return ElementSelection()



class DataGridLength(object,IEquatable[DataGridLength]):
 """
 Represents the lengths of elements within the System.Windows.Controls.DataGrid control.

 

 DataGridLength(pixels: float)

 DataGridLength(value: float,type: DataGridLengthUnitType)

 DataGridLength(value: float,type: DataGridLengthUnitType,desiredValue: float,displayValue: float)
 """
 def Equals(self,*__args):
  """
  Equals(self: DataGridLength,other: DataGridLength) -> bool

  

   Determines whether the specified System.Windows.Controls.DataGridLength is equal to the current 

    System.Windows.Controls.DataGridLength.

  

  

   other: The System.Windows.Controls.DataGridLength to compare to the current instance.

   Returns: true if the specified object is a System.Windows.Controls.DataGridLength with the same value or 

    sizing mode as the current System.Windows.Controls.DataGridLength; otherwise,false.

  

  Equals(self: DataGridLength,obj: object) -> bool

  

   Determines whether the specified object is equal to the current 

    System.Windows.Controls.DataGridLength.

  

  

   obj: The object to compare to the current instance.

   Returns: true if the specified object is a System.Windows.Controls.DataGridLength with the same value or 

    sizing mode as the current System.Windows.Controls.DataGridLength; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: DataGridLength) -> int

  

   Gets a hash code for the System.Windows.Controls.DataGridLength.

   Returns: A hash code for the current System.Windows.Controls.DataGridLength.
  """
  pass
 def ToString(self):
  """
  ToString(self: DataGridLength) -> str

  

   Returns a string that represents the current object.

   Returns: A string that represent the current object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,pixels: float)

  __new__(cls: type,value: float,type: DataGridLengthUnitType)

  __new__(cls: type,value: float,type: DataGridLengthUnitType,desiredValue: float,displayValue: float)

  __new__[DataGridLength]() -> DataGridLength
  """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 DesiredValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the calculated pixel value needed for the element.



Get: DesiredValue(self: DataGridLength) -> float



"""

 DisplayValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the pixel value allocated for the size of the element.



Get: DisplayValue(self: DataGridLength) -> float



"""

 IsAbsolute=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether this instance sizes elements based on a fixed pixel value.



Get: IsAbsolute(self: DataGridLength) -> bool



"""

 IsAuto=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether this instance automatically sizes elements based on both the content of cells and the column headers.



Get: IsAuto(self: DataGridLength) -> bool



"""

 IsSizeToCells=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether this instance automatically sizes elements based on the content of the cells.



Get: IsSizeToCells(self: DataGridLength) -> bool



"""

 IsSizeToHeader=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether this instance automatically sizes elements based on the header.



Get: IsSizeToHeader(self: DataGridLength) -> bool



"""

 IsStar=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether this instance automatically sizes elements based on a weighted proportion of available space.



Get: IsStar(self: DataGridLength) -> bool



"""

 UnitType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type that is used to determine how the size of the element is calculated.



Get: UnitType(self: DataGridLength) -> DataGridLengthUnitType



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the absolute value of the System.Windows.Controls.DataGridLength in pixels.



Get: Value(self: DataGridLength) -> float



"""


 Auto=None
 SizeToCells=None
 SizeToHeader=None




class BinaryWriter(object,IDisposable):
 """
 Writes primitive types in binary to a stream and supports writing strings in a specific encoding.

 

 BinaryWriter(output: Stream)

 BinaryWriter(output: Stream,encoding: Encoding)

 BinaryWriter(output: Stream,encoding: Encoding,leaveOpen: bool)
 """
 def Close(self):
  """
  Close(self: BinaryWriter)

   Closes the current System.IO.BinaryWriter and the underlying stream.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: BinaryWriter)

   Releases all resources used by the current instance of the System.IO.BinaryWriter class.
  """
  pass
 def Flush(self):
  """
  Flush(self: BinaryWriter)

   Clears all buffers for the current writer and causes any buffered data to be written to the 

    underlying device.
  """
  pass
 def Seek(self,offset,origin):
  """
  Seek(self: BinaryWriter,offset: int,origin: SeekOrigin) -> Int64

  

   Sets the position within the current stream.

  

   offset: A byte offset relative to origin.

   origin: A field of System.IO.SeekOrigin indicating the reference point from which the new position is to 

    be obtained.

  

   Returns: The position with the current stream.
  """
  pass
 def Write(self,*__args):
  """
  Write(self: BinaryWriter,value: UInt16)

   Writes a two-byte unsigned integer to the current stream and advances the stream position by two 

    bytes.

  

  

   value: The two-byte unsigned integer to write.

  Write(self: BinaryWriter,value: int)

   Writes a four-byte signed integer to the current stream and advances the stream position by four 

    bytes.

  

  

   value: The four-byte signed integer to write.

  Write(self: BinaryWriter,value: Decimal)

   Writes a decimal value to the current stream and advances the stream position by sixteen bytes.

  

   value: The decimal value to write.

  Write(self: BinaryWriter,value: Int16)

   Writes a two-byte signed integer to the current stream and advances the stream position by two 

    bytes.

  

  

   value: The two-byte signed integer to write.

  Write(self: BinaryWriter,value: UInt32)

   Writes a four-byte unsigned integer to the current stream and advances the stream position by 

    four bytes.

  

  

   value: The four-byte unsigned integer to write.

  Write(self: BinaryWriter,value: Single)

   Writes a four-byte floating-point value to the current stream and advances the stream position 

    by four bytes.

  

  

   value: The four-byte floating-point value to write.

  Write(self: BinaryWriter,value: str)

   Writes a length-prefixed string to this stream in the current encoding of the 

    System.IO.BinaryWriter,and advances the current position of the stream in accordance with the 

    encoding used and the specific characters being written to the stream.

  

  

   value: The value to write.

  Write(self: BinaryWriter,value: Int64)

   Writes an eight-byte signed integer to the current stream and advances the stream position by 

    eight bytes.

  

  

   value: The eight-byte signed integer to write.

  Write(self: BinaryWriter,value: UInt64)

   Writes an eight-byte unsigned integer to the current stream and advances the stream position by 

    eight bytes.

  

  

   value: The eight-byte unsigned integer to write.

  Write(self: BinaryWriter,value: SByte)

   Writes a signed byte to the current stream and advances the stream position by one byte.

  

   value: The signed byte to write.

  Write(self: BinaryWriter,buffer: Array[Byte])

   Writes a byte array to the underlying stream.

  

   buffer: A byte array containing the data to write.

  Write(self: BinaryWriter,value: bool)

   Writes a one-byte Boolean value to the current stream,with 0 representing false and 1 

    representing true.

  

  

   value: The Boolean value to write (0 or 1).

  Write(self: BinaryWriter,value: Byte)

   Writes an unsigned byte to the current stream and advances the stream position by one byte.

  

   value: The unsigned byte to write.

  Write(self: BinaryWriter,buffer: Array[Byte],index: int,count: int)

   Writes a region of a byte array to the current stream.

  

   buffer: A byte array containing the data to write.

   index: The starting point in buffer at which to begin writing.

   count: The number of bytes to write.

  Write(self: BinaryWriter,chars: Array[Char],index: int,count: int)

   Writes a section of a character array to the current stream,and advances the current position 

    of the stream in accordance with the Encoding used and perhaps the specific characters being 

    written to the stream.

  

  

   chars: A character array containing the data to write.

   index: The starting point in chars from which to begin writing.

   count: The number of characters to write.

  Write(self: BinaryWriter,value: float)

   Writes an eight-byte floating-point value to the current stream and advances the stream position 

    by eight bytes.

  

  

   value: The eight-byte floating-point value to write.

  Write(self: BinaryWriter,ch: Char)

   Writes a Unicode character to the current stream and advances the current position of the stream 

    in accordance with the Encoding used and the specific characters being written to the stream.

  

  

   ch: The non-surrogate,Unicode character to write.

  Write(self: BinaryWriter,chars: Array[Char])

   Writes a character array to the current stream and advances the current position of the stream 

    in accordance with the Encoding used and the specific characters being written to the stream.

  

  

   chars: A character array containing the data to write.
  """
  pass
 def Write7BitEncodedInt(self,*args):
  """
  Write7BitEncodedInt(self: BinaryWriter,value: int)

   Writes a 32-bit integer in a compressed format.

  

   value: The 32-bit integer to be written.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,output,encoding=None,leaveOpen=None):
  """
  __new__(cls: type)

  __new__(cls: type,output: Stream)

  __new__(cls: type,output: Stream,encoding: Encoding)

  __new__(cls: type,output: Stream,encoding: Encoding,leaveOpen: bool)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 BaseStream=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the underlying stream of the System.IO.BinaryWriter.



Get: BaseStream(self: BinaryWriter) -> Stream



"""


 Null=None
 OutStream=None




class EventDescriptorCollection(object,ICollection,IEnumerable,IList):
 """
 Represents a collection of System.ComponentModel.EventDescriptor objects.

 

 EventDescriptorCollection(events: Array[EventDescriptor])

 EventDescriptorCollection(events: Array[EventDescriptor],readOnly: bool)
 """
 def Add(self,value):
  """
  Add(self: EventDescriptorCollection,value: EventDescriptor) -> int

  

   Adds an System.ComponentModel.EventDescriptor to the end of the collection.

  

   value: An System.ComponentModel.EventDescriptor to add to the collection.

   Returns: The position of the System.ComponentModel.EventDescriptor within the collection.
  """
  pass
 def Clear(self):
  """
  Clear(self: EventDescriptorCollection)

   Removes all objects from the collection.
  """
  pass
 def Contains(self,value):
  """
  Contains(self: EventDescriptorCollection,value: EventDescriptor) -> bool

  

   Returns whether the collection contains the given System.ComponentModel.EventDescriptor.

  

   value: The System.ComponentModel.EventDescriptor to find within the collection.

   Returns: true if the collection contains the value parameter given; otherwise,false.
  """
  pass
 def Find(self,name,ignoreCase):
  """
  Find(self: EventDescriptorCollection,name: str,ignoreCase: bool) -> EventDescriptor

  

   Gets the description of the event with the specified name in the collection.

  

   name: The name of the event to get from the collection.

   ignoreCase: true if you want to ignore the case of the event; otherwise,false.

   Returns: The System.ComponentModel.EventDescriptor with the specified name,or null if the event does not 

    exist.
  """
  pass
 def GetEnumerator(self):
  """
  GetEnumerator(self: EventDescriptorCollection) -> IEnumerator

  

   Gets an enumerator for this System.ComponentModel.EventDescriptorCollection.

   Returns: An enumerator that implements System.Collections.IEnumerator.
  """
  pass
 def IndexOf(self,value):
  """
  IndexOf(self: EventDescriptorCollection,value: EventDescriptor) -> int

  

   Returns the index of the given System.ComponentModel.EventDescriptor.

  

   value: The System.ComponentModel.EventDescriptor to find within the collection.

   Returns: The index of the given System.ComponentModel.EventDescriptor within the collection.
  """
  pass
 def Insert(self,index,value):
  """
  Insert(self: EventDescriptorCollection,index: int,value: EventDescriptor)

   Inserts an System.ComponentModel.EventDescriptor to the collection at a specified index.

  

   index: The index within the collection in which to insert the value parameter.

   value: An System.ComponentModel.EventDescriptor to insert into the collection.
  """
  pass
 def InternalSort(self,*args):
  """
  InternalSort(self: EventDescriptorCollection,sorter: IComparer)

   Sorts the members of this System.ComponentModel.EventDescriptorCollection,using the specified 

    System.Collections.IComparer.

  

  

   sorter: A comparer to use to sort the System.ComponentModel.EventDescriptor objects in this collection.

  InternalSort(self: EventDescriptorCollection,names: Array[str])

   Sorts the members of this System.ComponentModel.EventDescriptorCollection. The specified order 

    is applied first,followed by the default sort for this collection,which is usually 

    alphabetical.

  

  

   names: An array of strings describing the order in which to sort the 

    System.ComponentModel.EventDescriptor objects in this collection.
  """
  pass
 def Remove(self,value):
  """
  Remove(self: EventDescriptorCollection,value: EventDescriptor)

   Removes the specified System.ComponentModel.EventDescriptor from the collection.

  

   value: The System.ComponentModel.EventDescriptor to remove from the collection.
  """
  pass
 def RemoveAt(self,index):
  """
  RemoveAt(self: EventDescriptorCollection,index: int)

   Removes the System.ComponentModel.EventDescriptor at the specified index from the collection.

  

   index: The index of the System.ComponentModel.EventDescriptor to remove.
  """
  pass
 def Sort(self,*__args):
  """
  Sort(self: EventDescriptorCollection,names: Array[str],comparer: IComparer) -> EventDescriptorCollection

  

   Sorts the members of this System.ComponentModel.EventDescriptorCollection,given a specified 

    sort order and an System.Collections.IComparer.

  

  

   names: An array of strings describing the order in which to sort the 

    System.ComponentModel.EventDescriptor objects in the collection.

  

   comparer: An System.Collections.IComparer to use to sort the System.ComponentModel.EventDescriptor objects 

    in this collection.

  

   Returns: The new System.ComponentModel.EventDescriptorCollection.

  Sort(self: EventDescriptorCollection,comparer: IComparer) -> EventDescriptorCollection

  

   Sorts the members of this System.ComponentModel.EventDescriptorCollection,using the specified 

    System.Collections.IComparer.

  

  

   comparer: An System.Collections.IComparer to use to sort the System.ComponentModel.EventDescriptor objects 

    in this collection.

  

   Returns: The new System.ComponentModel.EventDescriptorCollection.

  Sort(self: EventDescriptorCollection) -> EventDescriptorCollection

  

   Sorts the members of this System.ComponentModel.EventDescriptorCollection,using the default 

    sort for this collection,which is usually alphabetical.

  

   Returns: The new System.ComponentModel.EventDescriptorCollection.

  Sort(self: EventDescriptorCollection,names: Array[str]) -> EventDescriptorCollection

  

   Sorts the members of this System.ComponentModel.EventDescriptorCollection,given a specified 

    sort order.

  

  

   names: An array of strings describing the order in which to sort the 

    System.ComponentModel.EventDescriptor objects in the collection.

  

   Returns: The new System.ComponentModel.EventDescriptorCollection.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 @staticmethod
 def __new__(self,events,readOnly=None):
  """
  __new__(cls: type,events: Array[EventDescriptor])

  __new__(cls: type,events: Array[EventDescriptor],readOnly: bool)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of event descriptors in the collection.



Get: Count(self: EventDescriptorCollection) -> int



"""


 Empty=None




class TableLayoutStyle(object):
 """ Implements the basic functionality that represents the appearance and behavior of a table layout. """
 SizeType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a flag indicating how a row or column should be sized relative to its containing table.



Get: SizeType(self: TableLayoutStyle) -> SizeType



Set: SizeType(self: TableLayoutStyle)=value

"""





# encoding: utf-8
# module System.Web calls itself Web
# from System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class AspNetHostingPermission(CodeAccessPermission,IPermission,ISecurityEncodable,IStackWalk,IUnrestrictedPermission):
 """
 Controls access permissions in ASP.NET hosted environments. This class cannot be inherited.

 

 AspNetHostingPermission(state: PermissionState)

 AspNetHostingPermission(level: AspNetHostingPermissionLevel)
 """
 def Copy(self):
  """
  Copy(self: AspNetHostingPermission) -> IPermission

  

   When implemented by a derived class,creates and returns an identical copy of the current 

    permission object.

  

   Returns: A copy of the current permission object.
  """
  pass
 def FromXml(self,securityElement):
  """
  FromXml(self: AspNetHostingPermission,securityElement: SecurityElement)

   Reconstructs a permission object with a specified state from an XML encoding.

  

   securityElement: The System.Security.SecurityElement containing the XML encoding to use to reconstruct the 

    permission object.
  """
  pass
 def Intersect(self,target):
  """
  Intersect(self: AspNetHostingPermission,target: IPermission) -> IPermission

  

   When implemented by a derived class,creates and returns a permission that is the intersection 

    of the current permission and the specified permission.

  

  

   target: A permission to combine with the current permission. It must be of the same type as the current 

    permission.

  

   Returns: An System.Security.IPermission that represents the intersection of the current permission and 

    the specified permission; otherwise,null if the intersection is empty.
  """
  pass
 def IsSubsetOf(self,target):
  """
  IsSubsetOf(self: AspNetHostingPermission,target: IPermission) -> bool

  

   Returns a value indicating whether the current permission is a subset of the specified 

    permission.

  

  

   target: The System.Security.IPermission to combine with the current permission. It must be of the same 

    type as the current System.Security.IPermission.

  

   Returns: true if the current System.Security.IPermission is a subset of the specified 

    System.Security.IPermission; otherwise,false.
  """
  pass
 def IsUnrestricted(self):
  """
  IsUnrestricted(self: AspNetHostingPermission) -> bool

  

   Returns a value indicating whether unrestricted access to the resource that is protected by the 

    current permission is allowed.

  

   Returns: true if unrestricted use of the resource protected by the permission is allowed; otherwise,

    false.
  """
  pass
 def ToXml(self):
  """
  ToXml(self: AspNetHostingPermission) -> SecurityElement

  

   Creates an XML encoding of the permission object and its current state.

   Returns: A System.Security.SecurityElement containing the XML encoding of the permission object,

    including any state information.
  """
  pass
 def Union(self,target):
  """
  Union(self: AspNetHostingPermission,target: IPermission) -> IPermission

  

   Creates a permission that is the union of the current permission and the specified permission.

  

   target: A permission to combine with the current permission. It must be of the same type as the current 

    permission.

  

   Returns: An System.Security.IPermission that represents the union of the current permission and the 

    specified permission.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,state: PermissionState)

  __new__(cls: type,level: AspNetHostingPermissionLevel)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Level=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current hosting permission level for an ASP.NET application.



Get: Level(self: AspNetHostingPermission) -> AspNetHostingPermissionLevel



Set: Level(self: AspNetHostingPermission)=value

"""



class AspNetHostingPermissionAttribute(CodeAccessSecurityAttribute,_Attribute):
 """
 Allows security actions for System.Web.AspNetHostingPermission to be applied to code using declarative security. This class cannot be inherited.

 

 AspNetHostingPermissionAttribute(action: SecurityAction)
 """
 def CreatePermission(self):
  """
  CreatePermission(self: AspNetHostingPermissionAttribute) -> IPermission

  

   Creates a new System.Web.AspNetHostingPermission with the permission level previously set by the 

    System.Web.AspNetHostingPermissionAttribute.Level property.

  

   Returns: An System.Security.IPermission that is the new System.Web.AspNetHostingPermission.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,action):
  """ __new__(cls: type,action: SecurityAction) """
  pass
 def __reduce_ex__(self,*args):
  pass
 Level=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current hosting permission level.



Get: Level(self: AspNetHostingPermissionAttribute) -> AspNetHostingPermissionLevel



Set: Level(self: AspNetHostingPermissionAttribute)=value

"""



class AspNetHostingPermissionLevel(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the trust level that is granted to an ASP.NET Web application.

 

 enum AspNetHostingPermissionLevel,values: High (500),Low (300),Medium (400),Minimal (200),None (100),Unrestricted (600)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 High=None
 Low=None
 Medium=None
 Minimal=None
 None=None
 Unrestricted=None
 value__=None





class AssemblyLoadEventArgs(EventArgs):
 """
 Provides data for the System.AppDomain.AssemblyLoad event.

 

 AssemblyLoadEventArgs(loadedAssembly: Assembly)
 """
 @staticmethod
 def __new__(self,loadedAssembly):
  """ __new__(cls: type,loadedAssembly: Assembly) """
  pass
 LoadedAssembly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an System.Reflection.Assembly that represents the currently loaded assembly.



Get: LoadedAssembly(self: AssemblyLoadEventArgs) -> Assembly



"""





class NavigateEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.DataGrid.Navigate event.

 

 NavigateEventArgs(isForward: bool)
 """
 @staticmethod
 def __new__(self,isForward):
  """ __new__(cls: type,isForward: bool) """
  pass
 Forward=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether to navigate in a forward direction.



Get: Forward(self: NavigateEventArgs) -> bool



"""





class UCOMITypeInfo:
 """ Use System.Runtime.InteropServices.ComTypes.ITypeInfo instead. """
 def AddressOfMember(self,memid,invKind,ppv):
  """
  AddressOfMember(self: UCOMITypeInfo,memid: int,invKind: INVOKEKIND) -> IntPtr

  

   Retrieves the addresses of static functions or variables,such as those defined in a DLL.

  

   memid: Member ID of the static member's address to retrieve.

   invKind: Specifies whether the member is a property,and if so,what kind.
  """
  pass
 def CreateInstance(self,pUnkOuter,riid,ppvObj):
  """
  CreateInstance(self: UCOMITypeInfo,pUnkOuter: object,riid: Guid) -> (Guid,object)

  

   Creates a new instance of a type that describes a component class (coclass).

  

   pUnkOuter: Object which acts as the controlling IUnknown.

   riid: The IID of the interface that the caller will use to communicate with the resulting object.
  """
  pass
 def GetContainingTypeLib(self,ppTLB,pIndex):
  """
  GetContainingTypeLib(self: UCOMITypeInfo) -> (UCOMITypeLib,int)

  

   Retrieves the type library that contains this type description and its index within that type 

    library.
  """
  pass
 def GetDllEntry(self,memid,invKind,pBstrDllName,pBstrName,pwOrdinal):
  """
  GetDllEntry(self: UCOMITypeInfo,memid: int,invKind: INVOKEKIND) -> (str,str,Int16)

  

   Retrieves a description or specification of an entry point for a function in a DLL.

  

   memid: ID of the member function whose DLL entry description is to be returned.

   invKind: Specifies the kind of member identified by memid.
  """
  pass
 def GetDocumentation(self,index,strName,strDocString,dwHelpContext,strHelpFile):
  """
  GetDocumentation(self: UCOMITypeInfo,index: int) -> (str,str,int,str)

  

   Retrieves the documentation string,the complete Help file name and path,and the context ID for 

    the Help topic for a specified type description.

  

  

   index: ID of the member whose documentation is to be returned.
  """
  pass
 def GetFuncDesc(self,index,ppFuncDesc):
  """
  GetFuncDesc(self: UCOMITypeInfo,index: int) -> IntPtr

  

   Retrieves the System.Runtime.InteropServices.FUNCDESC structure that contains information about 

    a specified function.

  

  

   index: Index of the function description to return.
  """
  pass
 def GetIDsOfNames(self,rgszNames,cNames,pMemId):
  """
  GetIDsOfNames(self: UCOMITypeInfo,rgszNames: Array[str],cNames: int) -> Array[int]

  

   Maps between member names and member IDs,and parameter names and parameter IDs.

  

   rgszNames: On succesful return,an array of names to map.

   cNames: Count of names to map.
  """
  pass
 def GetImplTypeFlags(self,index,pImplTypeFlags):
  """
  GetImplTypeFlags(self: UCOMITypeInfo,index: int) -> int

  

   Retrieves the System.Runtime.InteropServices.IMPLTYPEFLAGS value for one implemented interface 

    or base interface in a type description.

  

  

   index: Index of the implemented interface or base interface.
  """
  pass
 def GetMops(self,memid,pBstrMops):
  """
  GetMops(self: UCOMITypeInfo,memid: int) -> str

  

   Retrieves marshaling information.

  

   memid: The member ID that indicates which marshaling information is needed.
  """
  pass
 def GetNames(self,memid,rgBstrNames,cMaxNames,pcNames):
  """
  GetNames(self: UCOMITypeInfo,memid: int,cMaxNames: int) -> (Array[str],int)

  

   Retrieves the variable with the specified member ID (or the name of the property or method and 

    its parameters) that correspond to the specified function ID.

  

  

   memid: The ID of the member whose name (or names) is to be returned.

   cMaxNames: Length of the rgBstrNames array.
  """
  pass
 def GetRefTypeInfo(self,hRef,ppTI):
  """
  GetRefTypeInfo(self: UCOMITypeInfo,hRef: int) -> UCOMITypeInfo

  

   If a type description references other type descriptions,it retrieves the referenced type 

    descriptions.

  

  

   hRef: Handle to the referenced type description to return.
  """
  pass
 def GetRefTypeOfImplType(self,index,href):
  """
  GetRefTypeOfImplType(self: UCOMITypeInfo,index: int) -> int

  

   If a type description describes a COM class,it retrieves the type description of the 

    implemented interface types.

  

  

   index: Index of the implemented type whose handle is returned.
  """
  pass
 def GetTypeAttr(self,ppTypeAttr):
  """
  GetTypeAttr(self: UCOMITypeInfo) -> IntPtr

  

   Retrieves a System.Runtime.InteropServices.TYPEATTR structure that contains the attributes of 

    the type description.
  """
  pass
 def GetTypeComp(self,ppTComp):
  """
  GetTypeComp(self: UCOMITypeInfo) -> UCOMITypeComp

  

   Retrieves the ITypeComp interface for the type description,which enables a client compiler to 

    bind to the type description's members.
  """
  pass
 def GetVarDesc(self,index,ppVarDesc):
  """
  GetVarDesc(self: UCOMITypeInfo,index: int) -> IntPtr

  

   Retrieves a VARDESC structure that describes the specified variable.

  

   index: Index of the variable description to return.
  """
  pass
 def Invoke(self,pvInstance,memid,wFlags,pDispParams,pVarResult,pExcepInfo,puArgErr):
  """
  Invoke(self: UCOMITypeInfo,pvInstance: object,memid: int,wFlags: Int16,pDispParams: DISPPARAMS) -> (DISPPARAMS,object,EXCEPINFO,int)

  

   Invokes a method,or accesses a property of an object,that implements the interface described 

    by the type description.

  

  

   pvInstance: Reference to the interface described by this type description.

   memid: Identifies the interface member.

   wFlags: Flags describing the context of the invoke call.

   pDispParams: Reference to a structure that contains an array of arguments,an array of DISPIDs for named 

    arguments,and counts of the number of elements in each array.
  """
  pass
 def ReleaseFuncDesc(self,pFuncDesc):
  """
  ReleaseFuncDesc(self: UCOMITypeInfo,pFuncDesc: IntPtr)

   Releases a System.Runtime.InteropServices.FUNCDESC previously returned by 

    System.Runtime.InteropServices.UCOMITypeInfo.GetFuncDesc(System.Int32,System.IntPtr@).

  

  

   pFuncDesc: Reference to the FUNCDESC to release.
  """
  pass
 def ReleaseTypeAttr(self,pTypeAttr):
  """
  ReleaseTypeAttr(self: UCOMITypeInfo,pTypeAttr: IntPtr)

   Releases a System.Runtime.InteropServices.TYPEATTR previously returned by 

    System.Runtime.InteropServices.UCOMITypeInfo.GetTypeAttr(System.IntPtr@).

  

  

   pTypeAttr: Reference to the TYPEATTR to release.
  """
  pass
 def ReleaseVarDesc(self,pVarDesc):
  """
  ReleaseVarDesc(self: UCOMITypeInfo,pVarDesc: IntPtr)

   Releases a VARDESC previously returned by 

    System.Runtime.InteropServices.UCOMITypeInfo.GetVarDesc(System.Int32,System.IntPtr@).

  

  

   pVarDesc: Reference to the VARDESC to release.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class SizeConverter(TypeConverter):
 """
 The System.Drawing.SizeConverter class is used to convert from one data type to another. Access this class through the System.ComponentModel.TypeDescriptor object.

 

 SizeConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: SizeConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Determines whether this converter can convert an object in the specified source type to the 

    native type of the converter.

  

  

   context: A System.ComponentModel.ITypeDescriptorContext that can be used to get additional information 

    about the environment this converter is being called from. This may be null,so you should 

    always check. Also,properties on the context object may also return null.

  

   sourceType: The type you want to convert from.

   Returns: This method returns true if this object can perform the conversion.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: SizeConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Gets a value indicating whether this converter can convert an object to the given destination 

    type using the context.

  

  

   context: A System.ComponentModel.ITypeDescriptorContext that can be used to get additional information 

    about the environment this converter is being called from. This can be null,so always check. 

    Also,properties on the context object can return null.

  

   destinationType: A System.Type that represents the type you want to convert to.

   Returns: This method returns true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: SizeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the specified object to the converter's native type.

  

   context: A System.ComponentModel.ITypeDescriptorContext that can be used to get additional information 

    about the environment this converter is being called from. This may be null,so you should 

    always check. Also,properties on the context object may also return null.

  

   culture: An System.Globalization.CultureInfo object that contains culture specific information,such as 

    the language,calendar,and cultural conventions associated with a specific culture. It is based 

    on the RFC 1766 standard.

  

   value: The object to convert.

   Returns: The converted object.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: SizeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the specified object to the specified type.

  

   context: A System.ComponentModel.ITypeDescriptorContext that can be used to get additional information 

    about the environment this converter is being called from. This may be null,so you should 

    always check. Also,properties on the context object may also return null.

  

   culture: An System.Globalization.CultureInfo object that contains culture specific information,such as 

    the language,calendar,and cultural conventions associated with a specific culture. It is based 

    on the RFC 1766 standard.

  

   value: The object to convert.

   destinationType: The type to convert the object to.

   Returns: The converted object.
  """
  pass
 def CreateInstance(self,*__args):
  """
  CreateInstance(self: SizeConverter,context: ITypeDescriptorContext,propertyValues: IDictionary) -> object

  

   Creates an object of this type by using a specified set of property values for the object. This 

    is useful for creating non-changeable objects that have changeable properties.

  

  

   context: A System.ComponentModel.TypeDescriptor through which additional context can be provided.

   propertyValues: A dictionary of new property values. The dictionary contains a series of name-value pairs,one 

    for each property returned from the 

    System.Drawing.SizeConverter.GetProperties(System.ComponentModel.ITypeDescriptorContext,System.Ob

    ject,System.Attribute[]) method.

  

   Returns: The newly created object,or null if the object could not be created. The default implementation 

    returns null.
  """
  pass
 def GetCreateInstanceSupported(self,context=None):
  """
  GetCreateInstanceSupported(self: SizeConverter,context: ITypeDescriptorContext) -> bool

  

   Determines whether changing a value on this object should require a call to the 

    System.Drawing.SizeConverter.CreateInstance(System.ComponentModel.ITypeDescriptorContext,System.C

    ollections.IDictionary) method to create a new value.

  

  

   context: A System.ComponentModel.TypeDescriptor through which additional context can be provided.

   Returns: true if the 

    System.Drawing.SizeConverter.CreateInstance(System.ComponentModel.ITypeDescriptorContext,System.C

    ollections.IDictionary) object should be called when a change is made to one or more properties 

    of this object.
  """
  pass
 def GetProperties(self,*__args):
  """
  GetProperties(self: SizeConverter,context: ITypeDescriptorContext,value: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   Retrieves the set of properties for this type. By default,a type does not have any properties 

    to return.

  

  

   context: A System.ComponentModel.TypeDescriptor through which additional context can be provided.

   value: The value of the object to get the properties for.

   attributes: An array of System.Attribute objects that describe the properties.

   Returns: The set of properties that should be exposed for this data type. If no properties should be 

    exposed,this may return null. The default implementation always returns null.
  """
  pass
 def GetPropertiesSupported(self,context=None):
  """
  GetPropertiesSupported(self: SizeConverter,context: ITypeDescriptorContext) -> bool

  

   Determines whether this object supports properties. By default,this is false.

  

   context: A System.ComponentModel.TypeDescriptor through which additional context can be provided.

   Returns: true if the 

    System.Drawing.SizeConverter.GetProperties(System.ComponentModel.ITypeDescriptorContext,System.Ob

    ject,System.Attribute[]) method should be called to find the properties of this object.
  """
  pass



class DataGridCheckBoxColumn(DataGridBoundColumn):
 """
 Represents a System.Windows.Controls.DataGrid column that hosts System.Windows.Controls.CheckBox controls in its cells.

 

 DataGridCheckBoxColumn()
 """
 DataGridOwner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Controls.DataGrid control that contains this column.



"""

 IsThreeState=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the hosted System.Windows.Controls.CheckBox controls enable three states or two.



Get: IsThreeState(self: DataGridCheckBoxColumn) -> bool



Set: IsThreeState(self: DataGridCheckBoxColumn)=value

"""


 DefaultEditingElementStyle=None
 DefaultElementStyle=None
 IsThreeStateProperty=None




class ToolBarButtonClickEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ToolBar.ButtonClick event.

 

 ToolBarButtonClickEventArgs(button: ToolBarButton)
 """
 @staticmethod
 def __new__(self,button):
  """ __new__(cls: type,button: ToolBarButton) """
  pass
 Button=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Forms.ToolBarButton that was clicked.



Get: Button(self: ToolBarButtonClickEventArgs) -> ToolBarButton



Set: Button(self: ToolBarButtonClickEventArgs)=value

"""





class ContextMenuEventArgs(RoutedEventArgs):
 """ Provides data for the context menu event. """
 CursorLeft=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the horizontal position of the mouse.



Get: CursorLeft(self: ContextMenuEventArgs) -> float



"""

 CursorTop=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the vertical position of the mouse.



Get: CursorTop(self: ContextMenuEventArgs) -> float



"""





class AccessibleRole(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies values representing possible roles for an accessible object.

 

 enum AccessibleRole,values: Alert (8),Animation (54),Application (14),Border (19),ButtonDropDown (56),ButtonDropDownGrid (58),ButtonMenu (57),Caret (7),Cell (29),Character (32),Chart (17),CheckButton (44),Client (10),Clock (61),Column (27),ColumnHeader (25),ComboBox (46),Cursor (6),Default (-1),Diagram (53),Dial (49),Dialog (18),Document (15),DropList (47),Equation (55),Graphic (40),Grip (4),Grouping (20),HelpBalloon (31),HotkeyField (50),Indicator (39),IpAddress (63),Link (30),List (33),ListItem (34),MenuBar (2),MenuItem (12),MenuPopup (11),None (0),Outline (35),OutlineButton (64),OutlineItem (36),PageTab (37),PageTabList (60),Pane (16),ProgressBar (48),PropertyPage (38),PushButton (43),RadioButton (45),Row (28),RowHeader (26),ScrollBar (3),Separator (21),Slider (51),Sound (5),SpinButton (52),SplitButton (62),StaticText (41),StatusBar (23),Table (24),Text (42),TitleBar (1),ToolBar (22),ToolTip (13),WhiteSpace (59),Window (9)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Alert=None
 Animation=None
 Application=None
 Border=None
 ButtonDropDown=None
 ButtonDropDownGrid=None
 ButtonMenu=None
 Caret=None
 Cell=None
 Character=None
 Chart=None
 CheckButton=None
 Client=None
 Clock=None
 Column=None
 ColumnHeader=None
 ComboBox=None
 Cursor=None
 Default=None
 Diagram=None
 Dial=None
 Dialog=None
 Document=None
 DropList=None
 Equation=None
 Graphic=None
 Grip=None
 Grouping=None
 HelpBalloon=None
 HotkeyField=None
 Indicator=None
 IpAddress=None
 Link=None
 List=None
 ListItem=None
 MenuBar=None
 MenuItem=None
 MenuPopup=None
 None=None
 Outline=None
 OutlineButton=None
 OutlineItem=None
 PageTab=None
 PageTabList=None
 Pane=None
 ProgressBar=None
 PropertyPage=None
 PushButton=None
 RadioButton=None
 Row=None
 RowHeader=None
 ScrollBar=None
 Separator=None
 Slider=None
 Sound=None
 SpinButton=None
 SplitButton=None
 StaticText=None
 StatusBar=None
 Table=None
 Text=None
 TitleBar=None
 ToolBar=None
 ToolTip=None
 value__=None
 WhiteSpace=None
 Window=None




class RectangleF(object):
 """
 Stores a set of four floating-point numbers that represent the location and size of a rectangle. For more advanced region functions,use a System.Drawing.Region object.

 

 RectangleF(x: Single,y: Single,width: Single,height: Single)

 RectangleF(location: PointF,size: SizeF)
 """
 def Contains(self,*__args):
  """
  Contains(self: RectangleF,rect: RectangleF) -> bool

  

   Determines if the rectangular region represented by rect is entirely contained within this 

    System.Drawing.RectangleF structure.

  

  

   rect: The System.Drawing.RectangleF to test.

   Returns: This method returns true if the rectangular region represented by rect is entirely contained 

    within the rectangular region represented by this System.Drawing.RectangleF; otherwise false.

  

  Contains(self: RectangleF,pt: PointF) -> bool

  

   Determines if the specified point is contained within this System.Drawing.RectangleF structure.

  

   pt: The System.Drawing.PointF to test.

   Returns: This method returns true if the point represented by the pt parameter is contained within this 

    System.Drawing.RectangleF structure; otherwise false.

  

  Contains(self: RectangleF,x: Single,y: Single) -> bool

  

   Determines if the specified point is contained within this System.Drawing.RectangleF structure.

  

   x: The x-coordinate of the point to test.

   y: The y-coordinate of the point to test.

   Returns: This method returns true if the point defined by x and y is contained within this 

    System.Drawing.RectangleF structure; otherwise false.
  """
  pass
 def Equals(self,obj):
  """
  Equals(self: RectangleF,obj: object) -> bool

  

   Tests whether obj is a System.Drawing.RectangleF with the same location and size of this 

    System.Drawing.RectangleF.

  

  

   obj: The System.Object to test.

   Returns: This method returns true if obj is a System.Drawing.RectangleF and its X,Y,Width,and Height 

    properties are equal to the corresponding properties of this System.Drawing.RectangleF; 

    otherwise,false.
  """
  pass
 @staticmethod
 def FromLTRB(left,top,right,bottom):
  """
  FromLTRB(left: Single,top: Single,right: Single,bottom: Single) -> RectangleF

  

   Creates a System.Drawing.RectangleF structure with upper-left corner and lower-right corner at 

    the specified locations.

  

  

   left: The x-coordinate of the upper-left corner of the rectangular region.

   top: The y-coordinate of the upper-left corner of the rectangular region.

   right: The x-coordinate of the lower-right corner of the rectangular region.

   bottom: The y-coordinate of the lower-right corner of the rectangular region.

   Returns: The new System.Drawing.RectangleF that this method creates.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: RectangleF) -> int

  

   Gets the hash code for this System.Drawing.RectangleF structure. For information about the use 

    of hash codes,see Object.GetHashCode.

  

   Returns: The hash code for this System.Drawing.RectangleF.
  """
  pass
 def Inflate(self,*__args):
  """
  Inflate(rect: RectangleF,x: Single,y: Single) -> RectangleF

  

   Creates and returns an enlarged copy of the specified System.Drawing.RectangleF structure. The 

    copy is enlarged by the specified amount and the original rectangle remains unmodified.

  

  

   rect: The System.Drawing.RectangleF to be copied. This rectangle is not modified.

   x: The amount to enlarge the copy of the rectangle horizontally.

   y: The amount to enlarge the copy of the rectangle vertically.

   Returns: The enlarged System.Drawing.RectangleF.

  Inflate(self: RectangleF,size: SizeF)

   Enlarges this System.Drawing.RectangleF by the specified amount.

  

   size: The amount to inflate this rectangle.

  Inflate(self: RectangleF,x: Single,y: Single)

   Enlarges this System.Drawing.RectangleF structure by the specified amount.

  

   x: The amount to inflate this System.Drawing.RectangleF structure horizontally.

   y: The amount to inflate this System.Drawing.RectangleF structure vertically.
  """
  pass
 def Intersect(self,*__args):
  """
  Intersect(a: RectangleF,b: RectangleF) -> RectangleF

  

   Returns a System.Drawing.RectangleF structure that represents the intersection of two 

    rectangles. If there is no intersection,and empty System.Drawing.RectangleF is returned.

  

  

   a: A rectangle to intersect.

   b: A rectangle to intersect.

   Returns: A third System.Drawing.RectangleF structure the size of which represents the overlapped area of 

    the two specified rectangles.

  

  Intersect(self: RectangleF,rect: RectangleF)

   Replaces this System.Drawing.RectangleF structure with the intersection of itself and the 

    specified System.Drawing.RectangleF structure.

  

  

   rect: The rectangle to intersect.
  """
  pass
 def IntersectsWith(self,rect):
  """
  IntersectsWith(self: RectangleF,rect: RectangleF) -> bool

  

   Determines if this rectangle intersects with rect.

  

   rect: The rectangle to test.

   Returns: This method returns true if there is any intersection.
  """
  pass
 def Offset(self,*__args):
  """
  Offset(self: RectangleF,x: Single,y: Single)

   Adjusts the location of this rectangle by the specified amount.

  

   x: The amount to offset the location horizontally.

   y: The amount to offset the location vertically.

  Offset(self: RectangleF,pos: PointF)

   Adjusts the location of this rectangle by the specified amount.

  

   pos: The amount to offset the location.
  """
  pass
 def ToString(self):
  """
  ToString(self: RectangleF) -> str

  

   Converts the Location and System.Drawing.Size of this System.Drawing.RectangleF to a 

    human-readable string.

  

   Returns: A string that contains the position,width,and height of this System.Drawing.RectangleF 

    structure. For example,"{X=20,Y=20,Width=100,Height=50}".
  """
  pass
 @staticmethod
 def Union(a,b):
  """
  Union(a: RectangleF,b: RectangleF) -> RectangleF

  

   Creates the smallest possible third rectangle that can contain both of two rectangles that form 

    a union.

  

  

   a: A rectangle to union.

   b: A rectangle to union.

   Returns: A third System.Drawing.RectangleF structure that contains both of the two rectangles that form 

    the union.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__[RectangleF]() -> RectangleF

  

  __new__(cls: type,x: Single,y: Single,width: Single,height: Single)

  __new__(cls: type,location: PointF,size: SizeF)
  """
  pass
 def __ne__(self,*args):
  pass
 Bottom=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the y-coordinate that is the sum of System.Drawing.RectangleF.Y and System.Drawing.RectangleF.Height of this System.Drawing.RectangleF structure.



Get: Bottom(self: RectangleF) -> Single



"""

 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the height of this System.Drawing.RectangleF structure.



Get: Height(self: RectangleF) -> Single



Set: Height(self: RectangleF)=value

"""

 IsEmpty=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Tests whether the System.Drawing.RectangleF.Width or System.Drawing.RectangleF.Height property of this System.Drawing.RectangleF has a value of zero.



Get: IsEmpty(self: RectangleF) -> bool



"""

 Left=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the x-coordinate of the left edge of this System.Drawing.RectangleF structure.



Get: Left(self: RectangleF) -> Single



"""

 Location=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the coordinates of the upper-left corner of this System.Drawing.RectangleF structure.



Get: Location(self: RectangleF) -> PointF



Set: Location(self: RectangleF)=value

"""

 Right=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the x-coordinate that is the sum of System.Drawing.RectangleF.X and System.Drawing.RectangleF.Width of this System.Drawing.RectangleF structure.



Get: Right(self: RectangleF) -> Single



"""

 Size=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the size of this System.Drawing.RectangleF.



Get: Size(self: RectangleF) -> SizeF



Set: Size(self: RectangleF)=value

"""

 Top=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the y-coordinate of the top edge of this System.Drawing.RectangleF structure.



Get: Top(self: RectangleF) -> Single



"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the width of this System.Drawing.RectangleF structure.



Get: Width(self: RectangleF) -> Single



Set: Width(self: RectangleF)=value

"""

 X=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the x-coordinate of the upper-left corner of this System.Drawing.RectangleF structure.



Get: X(self: RectangleF) -> Single



Set: X(self: RectangleF)=value

"""

 Y=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the y-coordinate of the upper-left corner of this System.Drawing.RectangleF structure.



Get: Y(self: RectangleF) -> Single



Set: Y(self: RectangleF)=value

"""


 Empty=None




class ICurrencyManagerProvider:
 """ Provides custom binding management for components. """
 def GetRelatedCurrencyManager(self,dataMember):
  """
  GetRelatedCurrencyManager(self: ICurrencyManagerProvider,dataMember: str) -> CurrencyManager

  

   Gets the System.Windows.Forms.CurrencyManager for this 

    System.Windows.Forms.ICurrencyManagerProvider and the specified data member.

  

  

   dataMember: The name of the column or list,within the data source,to obtain the 

    System.Windows.Forms.CurrencyManager for.

  

   Returns: The related System.Windows.Forms.CurrencyManager obtained from this 

    System.Windows.Forms.ICurrencyManagerProvider and the specified data member.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 CurrencyManager=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.CurrencyManager associated with this System.Windows.Forms.ICurrencyManagerProvider.



Get: CurrencyManager(self: ICurrencyManagerProvider) -> CurrencyManager



"""





class DragCompletedEventArgs(RoutedEventArgs):
 """
 Provides information about the System.Windows.Controls.Primitives.Thumb.DragCompleted event that occurs when a user completes a drag operation with the mouse of a System.Windows.Controls.Primitives.Thumb control.

 

 DragCompletedEventArgs(horizontalChange: float,verticalChange: float,canceled: bool)
 """
 @staticmethod
 def __new__(self,horizontalChange,verticalChange,canceled):
  """ __new__(cls: type,horizontalChange: float,verticalChange: float,canceled: bool) """
  pass
 Canceled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets whether the drag operation for a System.Windows.Controls.Primitives.Thumb was canceled by a call to the System.Windows.Controls.Primitives.Thumb.CancelDrag method.



Get: Canceled(self: DragCompletedEventArgs) -> bool



"""

 HorizontalChange=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the horizontal change in position of the System.Windows.Controls.Primitives.Thumb after the user drags the control with the mouse.



Get: HorizontalChange(self: DragCompletedEventArgs) -> float



"""

 VerticalChange=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the vertical change in position of the System.Windows.Controls.Primitives.Thumb after the user drags the control with the mouse.



Get: VerticalChange(self: DragCompletedEventArgs) -> float



"""





class MenuGlyph(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the image to draw when drawing a menu with the System.Windows.Forms.ControlPaint.DrawMenuGlyph(System.Drawing.Graphics,System.Drawing.Rectangle,System.Windows.Forms.MenuGlyph) method.

 

 enum MenuGlyph,values: Arrow (0),Bullet (2),Checkmark (1),Max (2),Min (0)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Arrow=None
 Bullet=None
 Checkmark=None
 Max=None
 Min=None
 value__=None




class UploadFileCompletedEventArgs(AsyncCompletedEventArgs):
 """ Provides data for the System.Net.WebClient.UploadFileCompleted event. """
 Result=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the server reply to a data upload operation that is started by calling an erload:System.Net.WebClient.UploadFileAsync method.



Get: Result(self: UploadFileCompletedEventArgs) -> Array[Byte]



"""





class UCOMIConnectionPoint:
 """ Use System.Runtime.InteropServices.ComTypes.IConnectionPoint instead. """
 def Advise(self,pUnkSink,pdwCookie):
  """
  Advise(self: UCOMIConnectionPoint,pUnkSink: object) -> int

  

   Establishes an advisory connection between the connection point and the caller's sink object.

  

   pUnkSink: Reference to the sink to receive calls for the outgoing interface managed by this connection 

    point.
  """
  pass
 def EnumConnections(self,ppEnum):
  """
  EnumConnections(self: UCOMIConnectionPoint) -> UCOMIEnumConnections

  

   Creates an enumerator object for iteration through the connections that exist to this connection 

    point.
  """
  pass
 def GetConnectionInterface(self,pIID):
  """
  GetConnectionInterface(self: UCOMIConnectionPoint) -> Guid

  

   Returns the IID of the outgoing interface managed by this connection point.
  """
  pass
 def GetConnectionPointContainer(self,ppCPC):
  """
  GetConnectionPointContainer(self: UCOMIConnectionPoint) -> UCOMIConnectionPointContainer

  

   Retrieves the IConnectionPointContainer interface pointer to the connectable object that 

    conceptually owns this connection point.
  """
  pass
 def Unadvise(self,dwCookie):
  """
  Unadvise(self: UCOMIConnectionPoint,dwCookie: int)

   Terminates an advisory connection previously established through 

    System.Runtime.InteropServices.UCOMIConnectionPoint.Advise(System.Object,System.Int32@).

  

  

   dwCookie: The connection cookie previously returned from 

    System.Runtime.InteropServices.UCOMIConnectionPoint.Advise(System.Object,System.Int32@).
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
